@@ -3,7 +3,7 @@
  * e x t e n d . h			-- All the stuff dealing with 
  *					   extended types
  *
- * Copyright (C) 1993,1994,1995 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright  1993-1996 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  * 
  *
  * Permission to use, copy, and/or distribute this software and its
@@ -20,7 +20,7 @@
  *
  *           Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date: 15-Mar-1995 11:36
- * Last file update:  5-Jun-1995 00:01
+ * Last file update: 26-Feb-1996 13:41
  */
 
 
@@ -30,6 +30,7 @@ SCM  STk_extended_apply(SCM x, SCM args, SCM env);
 void STk_extended_display(SCM x, SCM port, int mode);
 int  STk_extended_procedurep(SCM x);
 int  STk_extended_eval_parameters(SCM x);
+SCM  STk_extended_compare(SCM x, SCM y, int equalp);
 
 void STk_Cpointer_display(SCM obj, SCM port, int mode);
 
