@@ -2,7 +2,7 @@
  *
  *  p o r t . c			-- ports implementation
  *
- * Copyright (C) 1993,1994,1995 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright  1993-1996 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  * 
  *
  * Permission to use, copy, and/or distribute this software and its
@@ -19,51 +19,95 @@
  *
  *            Author: Erick Gallesio [eg@unice.fr]
  *    Creation date: 17-Feb-1993 12:27
- * Last file update: 16-Jul-1995 17:39
+ * Last file update: 13-Sep-1996 21:59
  *
  */
-#include <sys/ioctl.h>
-#include <sys/time.h>
-#include <ctype.h>
+#ifndef WIN32
+#  include <sys/ioctl.h>
+#  include <sys/time.h>
+#  include <ctype.h>
+#endif
+
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>	/* This seems to be useful only for AIX */
 #endif
+
+#ifndef NO_FD_SET
+#   define SELECT_MASK fd_set
+#else
+#   ifndef _AIX
+	typedef long fd_mask;
+#   endif
+#   if defined(_IBMR2)
+#	define SELECT_MASK void
+#   else
+#	define SELECT_MASK int
+#   endif
+#endif
+
 #include "stk.h"
 
+#ifdef WIN32
+  /* Provide substitute functions dor WIN32 */
+  FILE *popen(char *cmd, char *mode) 
+  {	
+    /* Returning NULL will yield an error */
+    return NULL;
+  }
+  void pclose(FILE *f)
+  {}
+#endif
+
 /* external vars */
 SCM STk_curr_iport, STk_curr_oport, STk_curr_eport, STk_eof_object;
 
-static SCM makeport(char *name, int type, int error)
+
+SCM STk_Cfile2port(char *name, FILE *f, int type, int flags)
 {
   SCM z;
+
+  NEWCELL(z, type);
+  z->storage_as.port.p   = (struct port_descr *) 
+    				must_malloc(sizeof(struct port_descr));
+  PORT_FILE(z)           = f;
+  PORT_FLAGS(z)	 	 = flags;
+  PORT_REVENT(z)	 = Ntruth;
+  PORT_WEVENT(z)	 = Ntruth;
+  PORT_NAME(z)           = (char *) must_malloc(strlen(name)+1);
+  strcpy(PORT_NAME(z), name);
+
+  return z;
+}
+
+static SCM makeport(char *name, int type, char *mode, int error)
+{
+  SCM z     = Ntruth;
+  int flags = 0;
   FILE *f;
-  long flag;
   char *full_name;
 
-  flag = No_interrupt(1);
-
+  STk_disallow_sigint();
   if (strncmp(name, "| ", 2)) {
     full_name = CHARS(STk_internal_expand_file_name(name));
 
-    if ((f = fopen(full_name,(type==tc_iport)? "r" : "w")) == NULL) {
+    if ((f = fopen(full_name, mode)) == NULL) {
       if (error) Err("could not open file", STk_makestring(name));
-      else return Ntruth;
+      else goto Out;
     }
   }
   else {
     full_name = name;
-    if ((f = popen(name+1,(type==tc_iport)? "r" : "w")) == NULL) {
+    if ((f = popen(name+1, mode)) == NULL) {
+      flags = PIPE_PORT;
       if (error) Err("could not create pipe", STk_makestring(name));
-      else return Ntruth;
+      else goto Out;
     }    
   }
 
-  NEWCELL(z, type);
-  z->storage_as.port.f    = f;
-  z->storage_as.port.name = (char *) must_malloc(strlen(full_name)+1);
-  strcpy(z->storage_as.port.name, full_name);
-  
-  No_interrupt(flag);
+  z = STk_Cfile2port(full_name, f, type, flags);
+
+Out:
+  STk_allow_sigint();
   return(z);
 }
   
@@ -74,7 +118,11 @@ static SCM verify_port(char *who, SCM port, int mode)
   if (port == UNBOUND)     /* test write 'cause of flush */
     port = (mode&F_WRITE) ? STk_curr_oport: STk_curr_iport; 
 
-  if (port->storage_as.port.f == NULL) {
+  if (!(INP(port) || OUTP(port))) {
+    sprintf(buff, "%s: bad port", who);
+    Err(buff, port);
+  }
+  if (PORT_FLAGS(port) & PORT_CLOSED) {
     sprintf(buff, "%s: port is closed", who);
     Err(buff, port);
   }
@@ -87,50 +135,45 @@ Error:
 
 static void closeport(SCM port)
 {
-  long flag;
+  if (PORT_FLAGS(port) & PORT_CLOSED) return;
 
-  flag = No_interrupt(1);
-  if (port->storage_as.port.f) {
-    if (ISPORTP(port) || OSPORTP(port))			  /* String port */
-      STk_free_string_port(port);
-    else
-      if (strncmp(port->storage_as.port.name, "| ",2)==0) /* Pipe port */
-	pclose(port->storage_as.port.f);
-      else						  /* File port */
-	fclose(port->storage_as.port.f);
-    port->storage_as.port.f = (FILE *) NULL;
+  STk_disallow_sigint();
+  
+  if (IPORTP(port) || OPORTP(port)) {			    /* Not a string port */
+#ifdef USE_TK
+    /* For pipe and file ports, delete the fileevent associated to it (if any) */
+    Tcl_DeleteFileHandler(Tcl_GetFile((ClientData) fileno(PORT_FILE(port)), 
+				    TCL_UNIX_FD));
+#endif
+    if (PORT_FLAGS(port) & PIPE_PORT)   		    /* Pipe port */
+      pclose(PORT_FILE(port));
+    else 						    /* File port */
+      fclose(PORT_FILE(port));
   }
-  No_interrupt(flag);
+  PORT_FLAGS(port) |= PORT_CLOSED;
+  STk_allow_sigint();
 }
 
 void STk_freeport(SCM port)
 {
-   long flag = No_interrupt(1);
-
-   closeport(port);
-   if (*(port->storage_as.port.name)) {
-     free(port->storage_as.port.name);
-     port->storage_as.port.name = "";
-   }
-   No_interrupt(flag);
+  STk_disallow_sigint();
+  closeport(port);
+  free(PORT_NAME(port));
+  free(port->storage_as.port.p);
+  STk_allow_sigint();
 }
 
 void STk_init_standard_ports(void)
 {
-  NEWCELL(STk_curr_iport, tc_iport);
-  STk_curr_iport->storage_as.port.name = "*stdin*"; 
-  STk_curr_iport->storage_as.port.f    = stdin;
+  STk_curr_iport = STk_Cfile2port("*stdin*",  STk_stdin, tc_iport, 0);  
   STk_gc_protect(&STk_curr_iport);
-
-  NEWCELL(STk_curr_oport, tc_oport);
-  STk_curr_oport->storage_as.port.name = "*stdout*"; 
-  STk_curr_oport->storage_as.port.f    = stdout;
+  
+  STk_curr_oport = STk_Cfile2port("*stdout*", STk_stdout, tc_oport, 0);
   STk_gc_protect(&STk_curr_oport);
- 
-  NEWCELL(STk_curr_eport, tc_oport);
-  STk_curr_eport->storage_as.port.name = "*stderr*"; 
-  STk_curr_eport->storage_as.port.f    = stderr;
+
+  STk_curr_eport = STk_Cfile2port("*stderr*", STk_stderr, tc_oport, 0);
   STk_gc_protect(&STk_curr_eport);
+
   
   NEWCELL(STk_eof_object, tc_eof);
   STk_gc_protect(&STk_eof_object);
@@ -149,7 +192,6 @@ static int do_load(char *full_name)
 {
   FILE *f;
   int c;
-  SCM previous_file_name;
 
   if (!STk_dirp(full_name)) {
      f = fopen(full_name, "r");
@@ -157,7 +199,7 @@ static int do_load(char *full_name)
      if (f == NULL) return 0;
 
      if (VCELL(Intern(LOAD_VERBOSE)) != Ntruth)
-       fprintf(stderr, ";; Loading file \"%s\"\n", full_name);
+       fprintf(STk_stderr, ";; Loading file \"%s\"\n", full_name);
      
      /* Just read one character. Assume that file is an object if this 
       * character is a control one. Here, I don't try to see if the file magic 
@@ -205,7 +247,7 @@ static int do_load(char *full_name)
        STk_line_counter	    = previous_line;
      }
      if (VCELL(Intern(LOAD_VERBOSE)) != Ntruth)
-       fprintf(stderr, ";; File \"%s\" loaded\n", full_name);
+       fprintf(STk_stderr, ";; File \"%s\" loaded\n", full_name);
      return 1;
   }
   /* No file found */
@@ -253,9 +295,17 @@ SCM STk_loadfile(char *fname, int err_if_absent)
   if (STk_llength(load_path)<0)     Err("load: bad loading path", load_path);
   if (STk_llength(load_suffixes)<0) Err("load: bad set of suffixes", load_suffixes);
  
+#ifdef WIN32
+  if ((len > 0 && (fname[0] == '/' || fname[0] == '\\' || fname[0] == '~')) ||
+      (len > 1 && fname[0] == '.' && (fname[1] == '/' || fname[1] == '\\')) ||
+      (len > 2 && fname[0] == '.' && fname[1] == '.' && (fname[2] == '/' ||
+							 fname[2]=='\\'))   ||
+      (len > 1 && isalpha(fname[0]) && fname[1]==':')) {
+#else
   if ((len > 0 && (fname[0] == '/' || fname[0] == '~')) ||
       (len > 1 && fname[0] == '.' && fname[1] == '/') ||
       (len > 2 && fname[0] == '.' && fname[1] == '.' && fname[2] == '/')) {
+#endif
     
     if (fname[0] == '~') 
       fname = CHARS(STk_internal_expand_file_name(fname));
@@ -320,7 +370,7 @@ PRIMITIVE STk_with_input_from_file(SCM string, SCM thunk)
 
   if ((k = setjmp(env)) == 0) {
     Top_jmp_buf     = &env;
-    STk_curr_iport  = makeport(CHARS(string), tc_iport, 1);
+    STk_curr_iport  = makeport(CHARS(string), tc_iport, "r", TRUE);
     result          = Apply(thunk, NIL);
   }
   /* restore normal error jmpbuf  and current input port*/
@@ -347,7 +397,7 @@ PRIMITIVE STk_with_output_to_file(SCM string, SCM thunk)
 
   if ((k = setjmp(env)) == 0) {
     Top_jmp_buf     = &env;
-    STk_curr_oport  = makeport(CHARS(string), tc_oport, 1);
+    STk_curr_oport  = makeport(CHARS(string), tc_oport, "w", TRUE);
     result          = Apply(thunk, NIL);
   }
   /* restore normal error jmpbuf  and current output port*/
@@ -364,13 +414,13 @@ PRIMITIVE STk_with_output_to_file(SCM string, SCM thunk)
 PRIMITIVE STk_open_input_file(SCM filename)
 {
   if (NSTRINGP(filename)) Err("open-input-file: bad file name", filename);
-  return makeport(CHARS(filename), tc_iport, TRUE);
+  return makeport(CHARS(filename), tc_iport, "r", TRUE);
 }
 
 PRIMITIVE STk_open_output_file(SCM filename)
 {
   if (NSTRINGP(filename)) Err("open-output-file: bad file name", filename);
-  return makeport(CHARS(filename), tc_oport, TRUE); 
+  return makeport(CHARS(filename), tc_oport, "w", TRUE); 
 }
 
 PRIMITIVE STk_close_input_port(SCM port)
@@ -392,7 +442,7 @@ PRIMITIVE STk_close_output_port(SCM port)
 PRIMITIVE STk_read(SCM port)
 {
   port = verify_port("read", port, F_READ);
-  return(STk_readf(port->storage_as.port.f, FALSE));
+  return(STk_readf(PORT_FILE(port), FALSE));
 }
 
 PRIMITIVE STk_read_char(SCM port)
@@ -400,7 +450,7 @@ PRIMITIVE STk_read_char(SCM port)
   int c;
 
   port = verify_port("read-char", port, F_READ);
-  c = Getc(port->storage_as.port.f);
+  c = Getc(PORT_FILE(port));
   return (c == EOF) ? STk_eof_object : STk_makechar(c);
 }
 
@@ -409,8 +459,8 @@ PRIMITIVE STk_peek_char(SCM port)
   int c;
 
   port = verify_port("peek-char", port, F_READ);
-  c = Getc(port->storage_as.port.f);
-  Ungetc(c, port->storage_as.port.f);
+  c = Getc(PORT_FILE(port));
+  Ungetc(c, PORT_FILE(port));
   return (c == EOF) ? STk_eof_object : STk_makechar(c);
 }
 
@@ -418,40 +468,28 @@ PRIMITIVE STk_eof_objectp(SCM obj)
 {
   return (obj == STk_eof_object)? Truth : Ntruth;
 }
-#ifdef max
-#undef max
-#endif
-#define max(a,b) ((a)>(b)? (a) : (b))
-
-#ifdef _STDIO_USES_IOSTREAM  /* GNU libc */
-#  if defined(_IO_STDIO_H) || defined (linux)
-#    define READ_DATA_PENDING(fp) (max(0,(fp)->_IO_read_end - (fp)->_IO_read_ptr))
-#  else
-#    define READ_DATA_PENDING(fp) (max(0,(fp)->_egptr - (fp)->_gptr))
-#  endif
-#endif
-#if (!defined (READ_DATA_PENDING)) && defined __SLBF
-#  define READ_DATA_PENDING(fp) (max(0,fp->_r))
-#endif
-#if !defined (READ_DATA_PENDING)
-#  define READ_DATA_PENDING(fp) (fp->_cnt)
-#endif
 
+#ifdef WIN32
+PRIMITIVE STk_char_readyp(SCM port) 
+{
+  STk_panic("Not yet implemented!");
+}
+#else
 PRIMITIVE STk_char_readyp(SCM port) 
 { 
   port = verify_port("char-ready?", port, F_READ);
-  if (Eof(port->storage_as.port.f)) return Truth;
+  if (Eof(PORT_FILE(port))) return Truth;
   if (ISPORTP(port)) /* !eof -> */  return Truth;
   else {
     /* First, see if characters are available in the buffer */
-    if (READ_DATA_PENDING(port->storage_as.port.f))
+    if (STk_file_data_available(PORT_FILE(port)))
       return Truth;
 
 #ifdef HAVE_SELECT
     {
-      fd_set readfds;
+      SELECT_MASK readfds;
       struct timeval timeout;
-      int f = fileno(port->storage_as.port.f);
+      int f = fileno(PORT_FILE(port));
 
       FD_ZERO(&readfds); 
       FD_SET(f, &readfds);
@@ -463,7 +501,7 @@ PRIMITIVE STk_char_readyp(SCM port)
    {
      int result;
 
-     ioctl(fileno(port->storage_as.port.f), FIONREAD, &result);
+     ioctl(fileno(PORT_FILE(port)), FIONREAD, &result);
      return result ? Truth : Ntruth;
    }
 #  else
@@ -472,7 +510,7 @@ PRIMITIVE STk_char_readyp(SCM port)
 #endif
   }
 }
-
+#endif
 
 PRIMITIVE STk_write(SCM expr, SCM port)
 {
@@ -491,7 +529,7 @@ PRIMITIVE STk_display(SCM expr, SCM port)
 PRIMITIVE STk_newline(SCM port)
 {
   port = verify_port("newline", port, F_WRITE);
-  Putc('\n', port->storage_as.port.f);
+  Putc('\n', PORT_FILE(port));
   return UNDEFINED;
 }
 
@@ -499,7 +537,7 @@ PRIMITIVE STk_write_char(SCM c, SCM port)
 {
   if (NCHARP(c)) Err("write-char: not a character", c);
   port = verify_port("write-char", port, F_WRITE);
-  Putc(CHAR(c), port->storage_as.port.f);
+  Putc(CHAR(c), PORT_FILE(port));
   return UNDEFINED;
 }
 
@@ -551,7 +589,7 @@ static SCM internal_format(SCM l,int len,int error)/* a very simple and poor one
   verify_port(error? "error": "format", port, F_WRITE);
   if (NSTRINGP(fmt)) Err("format: bad format string", fmt);
 
-  f = port->storage_as.port.f;
+  f = PORT_FILE(port);
 
   for(p=CHARS(fmt); *p; p++) {
     if (*p == '~') {
@@ -607,12 +645,19 @@ PRIMITIVE STk_try_load(SCM filename)
 PRIMITIVE STk_open_file(SCM filename, SCM mode)
 {
   int type;
+  
 
   if (NSTRINGP(filename)) Err("open-file: bad file name", filename); 
-  if (NSTRINGP(mode))     Err("open-file: bad mode", mode);
-
-  type = strchr(CHARS(mode), 'r') ? tc_iport : tc_oport;
-  return(makeport(CHARS(filename), type, FALSE));
+  if (NSTRINGP(mode) || CHARS(mode)[1] != '\0') goto Error;
+  
+  switch (CHARS(mode)[0]) {
+    case 'a': 
+    case 'w': type = tc_oport; break;
+    case 'r': type = tc_iport; break;
+    default:  ;
+Error:	      Err("open-file: bad mode", mode);
+  }
+  return(makeport(CHARS(filename), type, CHARS(mode), FALSE));
 }
 
 PRIMITIVE STk_close_port(SCM port)
@@ -630,7 +675,7 @@ PRIMITIVE STk_read_line(SCM port)
   SCM res;
 
   port = verify_port("read-line", port, F_READ);
-  f = port->storage_as.port.f;
+  f = PORT_FILE(port);
   for (i = 0; ; i++) {
     switch (c = Getc(f)) {
       case EOF:  if (i == 0) { free(buff); return STk_eof_object; }
@@ -646,8 +691,13 @@ PRIMITIVE STk_read_line(SCM port)
 
 PRIMITIVE STk_flush(SCM port)
 {
+  int code;
+
   port = verify_port("flush", port, F_WRITE|F_READ);
-  fflush(port->storage_as.port.f);
+  code = fflush(PORT_FILE(port));
+
+  if (code == EOF) Err("flush: cannot flush buffer", port);
+
   return UNDEFINED;
 }
 
@@ -715,3 +765,69 @@ PRIMITIVE STk_autoloadp(SCM l, SCM env, int len)
   
   return TYPEP(CAR(l), tc_autoload) ? Truth: Ntruth;
 }
+
+#ifdef USE_TK
+/******************************************************************************
+ *
+ * Port event management
+ *
+ ******************************************************************************/
+
+static void apply_file_closure(SCM closure)
+{
+  Apply(closure, NIL);
+}
+ 
+
+static SCM when_port_ready(SCM port, SCM closure, char *name, int mode)
+{
+  char str[50];
+  Tcl_File f;
+
+  if (NIPORTP(port) && NOPORTP(port)) {
+    sprintf(str, "%s: bad port", name);
+    STk_err(str, port);
+  }
+
+  if (closure == UNBOUND) {
+    /* Return the current handler closure */
+    return ((mode == TCL_READABLE)? PORT_REVENT(port): PORT_WEVENT(port));
+  }
+
+  f = Tcl_GetFile((ClientData) fileno(PORT_FILE(port)), TCL_UNIX_FD);
+  
+  if (closure == Ntruth) {
+    Tcl_DeleteFileHandler(f);    
+    if (mode == TCL_READABLE)
+      PORT_REVENT(port) = Ntruth;
+    else
+      PORT_WEVENT(port) = Ntruth;
+  }
+  else {
+    if (STk_procedurep(closure) == Ntruth) {
+      sprintf(str, "%s: bad closure", name);
+      STk_err(str, closure);
+    }
+
+    Tcl_CreateFileHandler(f, mode, (Tcl_FileProc *) apply_file_closure, 
+			  (ClientData) closure);
+    if (mode == TCL_READABLE)
+      PORT_REVENT(port) = closure;
+    else
+      PORT_WEVENT(port) = closure;
+  }
+  return UNDEFINED;
+}
+  
+
+PRIMITIVE STk_when_port_readable(SCM port, SCM closure)
+{
+  return when_port_ready(port, closure, "when-port-readable", TCL_READABLE);
+}
+
+
+PRIMITIVE STk_when_port_writable(SCM port, SCM closure)
+{
+  return when_port_ready(port, closure, "when-port-writable", TCL_WRITABLE);
+}
+#endif
