@@ -3,7 +3,7 @@
  *  p r o m i s e . c		-- Promises management
  *
  *
- * Copyright (C) 1993,1994,1995 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright  1993-1996 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  * 
  *
  * Permission to use, copy, and/or distribute this software and its
@@ -20,8 +20,7 @@
  *
  *            Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date:  2-Jun-1993 12:27
- * Last file update:  1-Feb-1995 20:02
- *
+ * Last file update: 10-Dec-1995 22:31
  *
  */
 
@@ -42,9 +41,18 @@ PRIMITIVE STk_force(SCM promise)
     return promise->storage_as.promise.expr;
 
   z = Apply(promise->storage_as.promise.expr, NIL);
-  promise->storage_as.promise.expr 	   = z;
-  promise->storage_as.promise.resultknown  = 1;
-  return z;
+
+  if (promise->storage_as.promise.resultknown)
+    /* R4RS: "A promise may refer to its own value.... Forcing such
+     * a promise may cause the promise to be forced a second time before
+     * the first value has been computed.
+     */
+    return promise->storage_as.promise.expr;
+  else {
+    promise->storage_as.promise.expr 	     = z;
+    promise->storage_as.promise.resultknown  = 1;
+    return z;
+  }
 }
 
 PRIMITIVE STk_promisep(SCM promise)
