@@ -6,7 +6,7 @@
  *				   Dummy functions are defined here to make the 
  *				   loader happy
  *
- * Copyright (C) 1993,1994,1995 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright  1993-1996 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  * 
  *
  * Permission to use, copy, and/or distribute this software and its
@@ -23,7 +23,7 @@
  *
  *           Author: Erick Gallesio [eg@unice.fr]
  *    Creation date:  1-Feb-1995 22:43
- * Last file update: 11-Feb-1995 21:19
+ * Last file update: 21-Sep-1996 22:38
  *
  ******************************************************************************/
 
@@ -31,12 +31,37 @@
 
 
 #ifndef USE_TK
+
+static Error(char *msg)
+{
+  char buffer[100];
+
+  sprintf(buffer, "Error: call of '%s' in Snow !!!. This should not occur\n", msg);
+}
+
 char *Tcl_SetVar2(interp, name1, name2, val, flags)
      Tcl_Interp *interp;
      char *name1, *name2, *val;
      int flags;
 {
-  Err("Fatal Error: call of Tcl_SetVar2 in snow !!!!!", NIL);
+  Error("Tcl_SetVar2");
+}
+
+Tcl_File Tcl_GetFile(ClientData fileData, int type)
+{
+  Error("Tcl_GetFile");
+  return (Tcl_File) TCL_ERROR;
+}
+
+void Tcl_DeleteFileHandler(Tcl_File file)
+{
+  Error("Tcl_DeleteFileHandler");
+}
+
+void Tcl_CreateFileHandler(Tcl_File file, int mask, Tcl_FileProc *proc,
+			    ClientData clientData)
+{
+  Error("Tcl_CreateFileHandler");
 }
 #endif
 
