@@ -17,7 +17,7 @@
  *
  *****
  *
- * Copyright (C) 1993, 1994 Erick Gallesio - I3S - CNRS / UNSA <eg@unice.fr>
+ * Copyright  1993-1996 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  * 
  *
  * Permission to use, copy, and/or distribute this software and its
@@ -34,7 +34,7 @@
  *
  *           Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date: 12-May-1993 10:34
- * Last file update:  6-Jun-1994 20:27
+ * Last file update: 25-Sep-1996 14:40
  */
 
 #include "stk.h"
@@ -42,7 +42,43 @@
 #define SMALLNUMP(x)	(fabs(x) <= SMALLINT_MAX)
 #define ISINT(x)	(floor(x) == (x))
 
-static SCM makesmallint(long n);
+/*  
+ * This interface has been added for the Frank Schmuelling complex-number
+ * package (Frank email is * <frank@zeus.ph1.Uni-Koeln.DE>). Don't rely on
+ * it, since rationnal and complex numbers will be present in a future
+ * release. As soon as the full tower will be implemented in STk, 
+ * this door will be closed.
+ */
+   
+#define do_addition 	_STk_do_addition
+#define do_substract 	_STk_do_substract
+#define do_multiply 	_STk_do_multiply
+#define do_divide 	_STk_do_divide
+#define bignum2double 	_STk_bignum2double
+
+
+static double do_compare(SCM x, SCM y);
+
+static SCM makesmallint(long x)
+{
+  SCM z;
+
+#ifndef COMPACT_SMALL_CST
+  NEWCELL(z, tc_integer);
+#endif
+  SET_INTEGER(z, x); 
+  return z;
+}
+
+static SCM makebignum(char *s)
+{
+  SCM z;
+  NEWCELL(z, tc_bignum); 
+  BIGNUM(z) = must_malloc(sizeof(MP_INT));
+  mpz_init_set_str(BIGNUM(z), s, 10);
+  return z;
+}
+
 
 /******************************************************************************
  * 
@@ -78,14 +114,19 @@ static double bignum2double(MP_INT *bn)
   return d;
 }
 
-static SCM double2bignum(double n)	/* Very inefficient */
+static SCM double2integer(double n)	/* small or big depending of n's size */
 {
   int i, j, size = 20;
   char *tmp = NULL;
   SCM z;
 
-  if (!ISINT(n)) err("cannot convert this number to an integer", makenumber(n));
+  if (!ISINT(n)) Err("cannot convert this number to an integer", STk_makenumber(n));
   
+  /* Try first to convert n to a long */
+  if (((double) SMALLINT_MIN <= n) && (n <= (double) SMALLINT_MAX))
+    return STk_makeinteger((long) n);
+
+  /* n doesn't fit in a long => build a bignum. THIS IS VERY INEFFICIENT */  
   tmp = must_malloc(size);
   i = 0;
   if (n < 0.0) { tmp[i++] = '-'; n = -n; }
@@ -113,13 +154,13 @@ static SCM double2bignum(double n)	/* Very inefficient */
 
 
 /* Convert a number to a C-string. Result must be freed if != from buffer */
-char *number2Cstr(SCM n, long base, char buffer[])
+char *STk_number2Cstr(SCM n, long base, char buffer[])
 {
   char *s = buffer;
   
   switch (TYPE(n)) {
     case tc_flonum:
-         if (base != 10) err("base must be 10 for this number", n);
+         if (base != 10) Err("base must be 10 for this number", n);
          sprintf(buffer, "%.15g", FLONM(n));
          if (strchr(buffer, '.') == NULL && strchr(buffer, 'e') == NULL) 
 	   strcat(buffer, ".0");
@@ -152,13 +193,14 @@ char *number2Cstr(SCM n, long base, char buffer[])
 	     return s;
 	   }
   }
-BadBase:
-  err("base must be 2, 8, 10 or 16", (base == LONG_MIN) ? NIL: makesmallint(base));
+
+  Err("base must be 2, 8, 10 or 16", (base == LONG_MIN) ? NIL: makesmallint(base));
+  return NULL; /* never reached */
 }
 
 /* Return #f if the C string doesn't contain a valid number. */
 /* UGLY! must be rewritten */
-SCM Cstr2number(char *str, long base)
+SCM STk_Cstr2number(char *str, long base)
 {
   int i, adigit=0, isint=1, exact=' ', radix = 0;
   char *start      = str;
@@ -168,13 +210,13 @@ SCM Cstr2number(char *str, long base)
     if (*p == '#') {
       p += 1;
       switch (*p++) {
-        case 'e': if (exact == ' ') { exact = 'e'; break; }  else return ntruth;
-	case 'i': if (exact == ' ') { exact = 'i'; break; }  else return ntruth;
-	case 'b': if (!radix) {base = 2;  radix = 1; break;} else return ntruth;
-	case 'o': if (!radix) {base = 8;  radix = 1; break;} else return ntruth;
-	case 'd': if (!radix) {base = 10; radix = 1; break;} else return ntruth;
-	case 'x': if (!radix) {base = 16; radix = 1; break;} else return ntruth;
-	default:  err("bad # syntax", makestrg(strlen(str), str));
+        case 'e': if (exact == ' ') { exact = 'e'; break; }  else return Ntruth;
+	case 'i': if (exact == ' ') { exact = 'i'; break; }  else return Ntruth;
+	case 'b': if (!radix) {base = 2;  radix = 1; break;} else return Ntruth;
+	case 'o': if (!radix) {base = 8;  radix = 1; break;} else return Ntruth;
+	case 'd': if (!radix) {base = 10; radix = 1; break;} else return Ntruth;
+	case 'x': if (!radix) {base = 16; radix = 1; break;} else return Ntruth;
+	default:  Err("bad # syntax", STk_makestring(str));
       }
       str += 2;
     }
@@ -182,7 +224,7 @@ SCM Cstr2number(char *str, long base)
   }
 
   if (base != 2 && base != 8 && base != 10 && base != 16)
-    err("base must be 2, 8, 10 or 16", (base == LONG_MIN) ? NIL: makesmallint(base));
+    Err("base must be 2, 8, 10 or 16", (base==LONG_MIN) ? NIL: makesmallint(base));
   
   if (*p == '-' || *p == '+') p+=1;
   if (*p == '#') goto End;
@@ -210,13 +252,17 @@ SCM Cstr2number(char *str, long base)
     MP_INT n;
 
     if (*str == '+') str+=1; /* mpz_init_set_str doesn't recognize +xyz !!! */
-    if (mpz_init_set_str(&n, str, base) < 0) return ntruth;
+    if (mpz_init_set_str(&n, str, base) < 0) {
+      mpz_clear(&n);
+      return Ntruth;
+    }
     if (mpz_cmp_si(&n, SMALLINT_MIN) >=0 && mpz_cmp_si(&n, SMALLINT_MAX) <= 0) {
       long num = mpz_get_si(&n);
-      return (exact == 'i') ? makenumber(num) : makesmallint(num);
+      mpz_clear(&n);
+      return (exact == 'i') ? STk_makenumber(num) : makesmallint(num);
     }
     /* It's a bignum */
-    if (exact == 'i') return makenumber(bignum2double(&n));
+    if (exact == 'i') return STk_makenumber(bignum2double(&n));
     else {
       SCM z;
 
@@ -230,7 +276,7 @@ SCM Cstr2number(char *str, long base)
   
   /* It's a float */
   if (exact == 'e')
-    err("#e cannot be specified on this number", makestrg(strlen(str), str));
+    Err("#e cannot be specified on this number", STk_makestring(str));
   if (base == 10) {
     /* Replace sharp signs by 0 */
     for(p=str; *p; p++) 
@@ -239,14 +285,14 @@ SCM Cstr2number(char *str, long base)
 	case 's': case 'S': case 'f': case 'F':
 	case 'd': case 'D': case 'l': case 'L': *p = 'e';
       }
-    return makenumber((double) atof(str));
+    return STk_makenumber((double) atof(str));
   }
 End:
-  if (*start ==  '#') err("Bad # syntax", makestrg(strlen(start), start));
-  return ntruth;
+  if (*start ==  '#') Err("Bad # syntax", STk_makestring(start));
+  return Ntruth;
 }
 
-long integer_value(SCM x) /* Returns LONG_MIN if not representable as int */
+long STk_integer_value(SCM x) /* Returns LONG_MIN if not representable as int */
 {
   if (INTEGERP(x)) return INTEGER(x);
   if (BIGNUMP(x)) {
@@ -257,6 +303,17 @@ long integer_value(SCM x) /* Returns LONG_MIN if not representable as int */
   return LONG_MIN;
 }
 
+long STk_integer_value_no_overflow(SCM x) /* Returns LONG_MIN if not an integer */
+{
+  if (INTEGERP(x)) return INTEGER(x);
+  if (BIGNUMP(x))  return mpz_get_si(BIGNUM(x));
+  return LONG_MIN;
+}
+
+int STk_equal_numbers(SCM number1, SCM number2) /* number1 = number2 */
+{
+  return do_compare(number1, number2) == 0;
+}
 
 /******************************************************************************
  *
@@ -270,15 +327,21 @@ static SCM clone(SCM number)
   switch (TYPE(number)) {
     case tc_integer: return makesmallint(INTEGER(number));
     case tc_bignum:  {
-      		       SCM c = makebignum("0");
-		       mpz_set(BIGNUM(c), BIGNUM(number));
-		       return c;
+      		       SCM z;
+		       
+		       NEWCELL(z, tc_bignum); 
+		       BIGNUM(z) = must_malloc(sizeof(MP_INT));
+		       mpz_init(BIGNUM(z));
+		       mpz_set(BIGNUM(z), BIGNUM(number));
+		       return z;
 		     }
-    case tc_flonum:  return makenumber(FLONM(number));
+    case tc_flonum:  return STk_makenumber(FLONM(number));
+    default:	     /* This is not a number. Return the parameter unmodified. */
+                     return number; /* Error will be signaled later */
   }
 }
 
-static void do_multiply(SCM *x, SCM y)
+void _STk_do_multiply(SCM *x, SCM y)
 {
   switch (TYPE(*x)) {
     case tc_integer:
@@ -299,7 +362,7 @@ static void do_multiply(SCM *x, SCM y)
 		mpz_mul(BIGNUM(*x), BIGNUM(*x), BIGNUM(y));
 		break;
 	   case tc_flonum:
-		*x = makenumber(INTEGER(*x) * FLONM(y));
+		*x = STk_makenumber(INTEGER(*x) * FLONM(y));
 		break;
 	 }
 	 break;
@@ -315,7 +378,7 @@ static void do_multiply(SCM *x, SCM y)
 		mpz_mul(BIGNUM(*x), BIGNUM(*x),  BIGNUM(y));
 		break;
 	   case tc_flonum:
-		*x = makenumber(bignum2double(BIGNUM(*x)) * FLONM(y));
+		*x = STk_makenumber(bignum2double(BIGNUM(*x)) * FLONM(y));
 		break;
 	 }
 	 break;
@@ -335,7 +398,7 @@ static void do_multiply(SCM *x, SCM y)
   }
 }
 
-static void do_addition(SCM *x, SCM y)
+void _STk_do_addition(SCM *x, SCM y)
 {
   switch (TYPE(*x)) {
     case tc_integer:
@@ -356,7 +419,7 @@ static void do_addition(SCM *x, SCM y)
 		mpz_add(BIGNUM(*x), BIGNUM(*x), BIGNUM(y));
 		break;
 	   case tc_flonum:
-		*x = makenumber(INTEGER(*x) + FLONM(y));
+		*x = STk_makenumber(INTEGER(*x) + FLONM(y));
 		break;
 	 }
 	 break;
@@ -372,7 +435,7 @@ static void do_addition(SCM *x, SCM y)
 		mpz_add(BIGNUM(*x), BIGNUM(*x),  BIGNUM(y));
 		break;
 	   case tc_flonum:
-		*x = makenumber(bignum2double(BIGNUM(*x)) + FLONM(y));
+		*x = STk_makenumber(bignum2double(BIGNUM(*x)) + FLONM(y));
 		break;
 	 }
 	 break;
@@ -392,7 +455,7 @@ static void do_addition(SCM *x, SCM y)
   }
 }
 
-static void do_substract(SCM *x, SCM y)
+static void _STk_do_substract(SCM *x, SCM y)
 {
   switch (TYPE(*x)) {
     case tc_integer:
@@ -413,7 +476,7 @@ static void do_substract(SCM *x, SCM y)
 		mpz_sub(BIGNUM(*x), BIGNUM(*x), BIGNUM(y));
 		break;
 	   case tc_flonum:
-		*x = makenumber(INTEGER(*x) - FLONM(y));
+		*x = STk_makenumber(INTEGER(*x) - FLONM(y));
 		break;
 	 }
 	 break;
@@ -429,7 +492,7 @@ static void do_substract(SCM *x, SCM y)
 		mpz_sub(BIGNUM(*x), BIGNUM(*x),  BIGNUM(y));
 		break;
 	   case tc_flonum:
-		*x = makenumber(bignum2double(BIGNUM(*x)) - FLONM(y));
+		*x = STk_makenumber(bignum2double(BIGNUM(*x)) - FLONM(y));
 		break;
 	 }
 	 break;
@@ -449,7 +512,7 @@ static void do_substract(SCM *x, SCM y)
   }
 }
 
-static void do_divide(SCM *x, SCM y)
+static void _STk_do_divide(SCM *x, SCM y)
 {
   switch (TYPE(*x)) {
     case tc_integer:
@@ -461,7 +524,7 @@ static void do_divide(SCM *x, SCM y)
 		  if (ISINT(div))
 		    SET_INTEGER(*x, (double) div);
 		  else
-		    *x = makenumber(div);
+		    *x = STk_makenumber(div);
 		}
 		break;
 	   case tc_bignum:
@@ -474,13 +537,13 @@ static void do_divide(SCM *x, SCM y)
 		  if (mpz_cmp_ui(&r, 0L) == 0)
 		    mpz_set(BIGNUM(*x), &q);
 		  else
-		    *x=makenumber(bignum2double(BIGNUM(*x)) /
-				  bignum2double(BIGNUM(y)));
+		    *x=STk_makenumber(bignum2double(BIGNUM(*x)) /
+				      bignum2double(BIGNUM(y)));
 		  mpz_clear(&q); mpz_clear(&r);
 		}
 		break;
 	   case tc_flonum:
-		*x = makenumber(INTEGER(*x) / FLONM(y));
+		*x = STk_makenumber(INTEGER(*x) / FLONM(y));
 		break;
 	 }
 	 break;
@@ -499,13 +562,13 @@ static void do_divide(SCM *x, SCM y)
 		if (mpz_cmp_ui(&r, 0L) == 0)
 		  mpz_set(BIGNUM(*x), &q);
 		else
-		  *x=makenumber(bignum2double(BIGNUM(*x)) /
-				bignum2double(BIGNUM(y)));
+		  *x=STk_makenumber(bignum2double(BIGNUM(*x)) /
+				    bignum2double(BIGNUM(y)));
 		mpz_clear(&q); mpz_clear(&r);
 	      }
 	      break;
 	   case tc_flonum:
-		*x = makenumber(bignum2double(BIGNUM(*x)) / FLONM(y));
+		*x = STk_makenumber(bignum2double(BIGNUM(*x)) / FLONM(y));
 		break;
 	 }
 	 break;
@@ -527,41 +590,45 @@ static void do_divide(SCM *x, SCM y)
 
 static SCM do_integer_division(SCM x, SCM y, int quotientp)
 {
+  MP_INT q, r;
+  SCM res;
+  int exact = 1;
+
   /* Divide x by y and return its quotient or remainder. y is not 0 */
-  if (INTEGERP(x) && INTEGERP(y)) {
-    return makesmallint(quotientp? INTEGER(x)/INTEGER(y):
-		       		  INTEGER(x)%INTEGER(y));
-  }
+  if (FLONUMP(x))
+    if (ISINT(FLONM(x))) { x = double2integer(FLONM(x)); exact = 0; }
+    else Err("bad number in an integer division", x);
+  
+  if (FLONUMP(y))
+    if (ISINT(FLONM(y))) { y = double2integer(FLONM(y)); exact = 0; }
+    else Err("bad number in an integer division", y);
+  
+  /* Here, x and y can only be integer or bignum (not float) */
+  if (INTEGERP(x))
+    if (INTEGERP(y)) {
+      res = makesmallint(quotientp? INTEGER(x)/INTEGER(y):
+			  	     INTEGER(x)%INTEGER(y));
+      return exact? res : STk_exact2inexact(res);
+    }
+    else
+      x = int2bignum(INTEGER(x));
   else {
-    /* result is a bignum. Eventually, transform argument to bignum  */
-    MP_INT q,  r;
-    SCM res   = makebignum("0");
-    int exact = 1;
-
-    switch (TYPE(x)) {
-      case tc_integer: x = int2bignum(INTEGER(x)); break;
-      case tc_bignum:  break;
-      case tc_flonum:  if (ISINT(FLONM(x))) {x=double2bignum(FLONM(x)); exact=0;}
-      		       else goto Error;
-    }
-    switch (TYPE(y)) {
-      case tc_integer: y = int2bignum(INTEGER(y)); break;
-      case tc_bignum:  break;
-      case tc_flonum:  if (ISINT(FLONM(y))) {y=double2bignum(FLONM(y)); exact=0;}
-      		       else { x = y; goto Error;}
-    }
-
-    mpz_init(&q); mpz_init(&r);
-    mpz_mdivmod(&q, &r, BIGNUM(x), BIGNUM(y));
-    mpz_set(BIGNUM(res), quotientp? &q : &r);
-    mpz_clear(&q); mpz_clear(&r);
-    return exact? res : exact2inexact(res);
+    /* x is a bignum */
+    if (INTEGERP(y))
+      y = int2bignum(INTEGER(y));
   }
-Error:
-  err("bad number in an integer division", x);
+  
+  /* x and y are both bignums */
+  res = makebignum("0");
+
+  mpz_init(&q); mpz_init(&r);
+  mpz_divmod(&q, &r, BIGNUM(x), BIGNUM(y)); /* use divmod instead of mdivmod */
+  mpz_set(BIGNUM(res), quotientp? &q : &r); /* Thanks to kerch@parc.xerox.com*/
+  mpz_clear(&q); mpz_clear(&r);
+  return exact? res : STk_exact2inexact(res);
 }
 
-double do_compare(SCM x, SCM y)
+static double do_compare(SCM x, SCM y)
 {
   switch (TYPE(x)) {
     case tc_integer:
@@ -582,19 +649,10 @@ double do_compare(SCM x, SCM y)
 	   case tc_bignum:  return (FLONM(x) - bignum2double(BIGNUM(y)));
 	   case tc_flonum:  return (FLONM(x) - FLONM(y));
 	 }
+    default: return 0.0; /* never reached */
   }
 }
 
-static SCM makesmallint(long x)
-{
-  SCM z;
-
-#ifndef COMPACT_SMALL_CST
-  NEWCELL(z, tc_integer);
-#endif
-  SET_INTEGER(z, x); 
-  return z;
-}
 
 /******************************************************************************
  * 
@@ -602,388 +660,431 @@ static SCM makesmallint(long x)
  *
  ******************************************************************************/
 
-SCM makenumber(double x)
+SCM STk_makenumber(double x)
 {
+  /* Floats are not stored in a struct obj since this leads to memory consumption
+   * This  memory consumption due to alignment problems. 
+   * For instance on a Sun 4, where double are 8 bytes, a struct obj with a double
+   * in line will occupy 16 bytes whereas it occupies only 12 bytes if the double
+   * is mallocated.
+   * This change (94/08/29) will give worst performances when crunching numbers,
+   * but use Fortran if this is your job :->
+   */
+
   SCM z;
-  NEWCELL(z,tc_flonum); FLONM(z) = x; return z;
+  NEWCELL(z,tc_flonum); 
+  (*z).storage_as.flonum.data = must_malloc(sizeof(double));
+  FLONM(z) = x; 
+  return z;
 }
 
 
-SCM makeinteger(long x)
+SCM STk_makeinteger(long x)
 {
   return (SMALLINT_MIN <= x && x <= SMALLINT_MAX) ? makesmallint(x): int2bignum(x);
 }
     
 
-SCM makebignum(char *s)
-{
-  SCM z;
-  NEWCELL(z, tc_bignum); 
-  BIGNUM(z) = must_malloc(sizeof(MP_INT));
-  mpz_init_set_str(BIGNUM(z), s, 10);
-  return z;
-}
-
 /******************************************************************************/
 
 /**** Section 6.5 ****/
 
-PRIMITIVE numberp(SCM x)
+PRIMITIVE STk_numberp(SCM x)
 {
-  return NUMBERP(x)? truth : ntruth;
+  return NUMBERP(x)? Truth : Ntruth;
 }
 
-PRIMITIVE integerp(SCM x)
+PRIMITIVE STk_integerp(SCM x)
 {
-  return (INTEGERP(x) || BIGNUMP(x)) ? truth : ntruth;
+  switch (TYPE(x)) {
+    case tc_integer:
+    case tc_bignum: return Truth;
+    case tc_flonum: {
+      		      double val = FLONM(x);
+		      return (floor(val) == val)? Truth : Ntruth;
+		    }
+    default: return Ntruth;
+  }
 }
 
-PRIMITIVE exactp(SCM x)
+PRIMITIVE STk_exactp(SCM x)
 {
-  if (NNUMBERP(x)) err("exact?: Bad number", x);
-  return EXACTP(x) ? truth: ntruth;
+  if (NNUMBERP(x)) Err("exact?: Bad number", x);
+  return EXACTP(x) ? Truth: Ntruth;
 }
 
-PRIMITIVE inexactp(SCM x)
+PRIMITIVE STk_inexactp(SCM x)
 {
-  if (NNUMBERP(x)) err("inexact?: Bad number", x);
-  return NEXACTP(x) ? truth: ntruth;
+  if (NNUMBERP(x)) Err("inexact?: Bad number", x);
+  return NEXACTP(x) ? Truth: Ntruth;
 }
 
 
 #define Compare(name, operator)					\
-PRIMITIVE name(SCM l, SCM env)					\
+PRIMITIVE name(SCM l, SCM env, int from_eval)			\
 {								\
   register SCM tmp1, tmp2;					\
 								\
-  if (NCONSP(l)) err("too few parameters", l);			\
-  tmp1 = EVALCAR(l);						\
+  if (NCONSP(l)) Err("too few parameters", l);			\
+  tmp1 = from_eval? EVALCAR(l): CAR(l);				\
   if (NNUMBERP(tmp1)) goto Error;				\
 								\
   for (l=CDR(l); NNULLP(l); l=CDR(l),tmp1=tmp2) {		\
-    tmp2 = EVALCAR(l);						\
+    tmp2 = from_eval? EVALCAR(l): CAR(l);			\
     if (NNUMBERP(tmp2)) goto Error;				\
-    if (do_compare(tmp1, tmp2) operator 0) return ntruth;	\
+    if (do_compare(tmp1, tmp2) operator 0) return Ntruth;	\
   }								\
-  return truth;							\
+  return Truth;							\
 Error:								\
-  err("Bad number in a comparison", l);				\
+  Err("Bad number in a comparison", l);				\
+  return UNDEFINED; /* never reached */				\
 }
 
-Compare(numequal,  !=)
-Compare(lessp,     >=)
-Compare(greaterp,  <=)
-Compare(lessep,    >)
-Compare(greaterep, <)
+Compare(STk_numequal,  !=)
+Compare(STk_lessp,     >=)
+Compare(STk_greaterp,  <=)
+Compare(STk_lessep,    >)
+Compare(STk_greaterep, <)
 
 
-PRIMITIVE zerop(SCM n)
+PRIMITIVE STk_zerop(SCM n)
 {
   switch (TYPE(n)) {
-    case tc_integer: return (INTEGER(n) == 0) 		     ? truth: ntruth;
-    case tc_flonum:  return (FLONM(n) == 0.0) 		     ? truth: ntruth;
-    case tc_bignum:  return (mpz_cmp_ui(BIGNUM(n), 0L) == 0) ? truth: ntruth;
-    default:         err("zero?: bad number", n);
+    case tc_integer: return (INTEGER(n) == 0) 		     ? Truth: Ntruth;
+    case tc_flonum:  return (FLONM(n) == 0.0) 		     ? Truth: Ntruth;
+    case tc_bignum:  return (mpz_cmp_ui(BIGNUM(n), 0L) == 0) ? Truth: Ntruth;
+    default:         Err("zero?: bad number", n); 
+		     return UNDEFINED; /* never reached */
   }
 }
 
-PRIMITIVE positivep(SCM n)
+PRIMITIVE STk_positivep(SCM n)
 {
   switch (TYPE(n)) {
-    case tc_integer: return (INTEGER(n) > 0) 		     ? truth: ntruth;
-    case tc_flonum:  return (FLONM(n) > 0.0) 		     ? truth: ntruth;
-    case tc_bignum:  return (mpz_cmp_ui(BIGNUM(n), 0L) > 0)  ? truth: ntruth;
-    default:         err("positive?: bad number", n);
+    case tc_integer: return (INTEGER(n) > 0) 		     ? Truth: Ntruth;
+    case tc_flonum:  return (FLONM(n) > 0.0) 		     ? Truth: Ntruth;
+    case tc_bignum:  return (mpz_cmp_ui(BIGNUM(n), 0L) > 0)  ? Truth: Ntruth;
+    default:         Err("positive?: bad number", n);
+		     return UNDEFINED; /* never reached */
   }
 }  
 
-PRIMITIVE negativep(SCM n)
+PRIMITIVE STk_negativep(SCM n)
 {
   switch (TYPE(n)) {
-    case tc_integer: return (INTEGER(n) < 0) 		     ? truth: ntruth;
-    case tc_flonum:  return (FLONM(n) < 0.0) 		     ? truth: ntruth;
-    case tc_bignum:  return (mpz_cmp_ui(BIGNUM(n), 0L) < 0)  ? truth: ntruth;
-    default:         err("negative?: bad number", n);
+    case tc_integer: return (INTEGER(n) < 0) 		     ? Truth: Ntruth;
+    case tc_flonum:  return (FLONM(n) < 0.0) 		     ? Truth: Ntruth;
+    case tc_bignum:  return (mpz_cmp_ui(BIGNUM(n), 0L) < 0)  ? Truth: Ntruth;
+    default:         Err("negative?: bad number", n);
+		     return UNDEFINED; /* never reached */
   }
 }
 
-PRIMITIVE oddp(SCM n)
+PRIMITIVE STk_oddp(SCM n)
 {
   MP_INT q, r;
   long res;
 
   switch (TYPE(n)) {
-    case tc_integer: return (INTEGER(n)%2) 		     ? truth: ntruth;
+    case tc_integer: return (INTEGER(n)%2) 		     ? Truth: Ntruth;
     case tc_bignum:  mpz_init(&q), mpz_init(&r);
       		     mpz_divmod_ui(&q, &r, BIGNUM(n), 2L);
       		     res = mpz_cmp_ui(&r, 0L);
       		     mpz_clear(&q); mpz_clear(&r);
-      		     return (res != 0) ? truth: ntruth;
-    default:         err("odd?: bad number", n);
+      		     return (res != 0) ? Truth: Ntruth;
+    default:         Err("odd?: bad number", n);
+		     return UNDEFINED; /* never reached */
   }
 }  
 
-PRIMITIVE evenp(SCM n)
+PRIMITIVE STk_evenp(SCM n)
 {
   MP_INT q, r;
   long res;
 
   switch (TYPE(n)) {
-    case tc_integer: return (INTEGER(n)%2) 		     ? ntruth: truth;
+    case tc_integer: return (INTEGER(n)%2) 		     ? Ntruth: Truth;
     case tc_bignum:  mpz_init(&q), mpz_init(&r);
       		     mpz_divmod_ui(&q, &r, BIGNUM(n), 2L);
       		     res = mpz_cmp_ui(&r, 0L);
       		     mpz_clear(&q); mpz_clear(&r);
-      		     return (res == 0) ? truth: ntruth;
-    default:         err("even?: bad number", n);
+      		     return (res == 0) ? Truth: Ntruth;
+    default:         Err("even?: bad number", n);
+		     return UNDEFINED; /* never reached */
   }
 }
 
-PRIMITIVE lmax(SCM l, SCM env)
+PRIMITIVE STk_max(SCM l, SCM env, int from_eval)
 {
   register SCM tmp, max;
   int inexact;
 
-  if (NULLP(l)) err("max: bad number of arguments", NIL);
-  
-  if (NNUMBERP(tmp=EVALCAR(l))) goto Error;
+  if (NULLP(l)) Err("max: bad number of arguments", NIL);
+
+  tmp = from_eval? EVALCAR(l): CAR(l);
+  if (NNUMBERP(tmp)) goto Error;
   inexact = FLONUMP(tmp);
   max = tmp;
   
   for (l=CDR(l); NNULLP(l); l=CDR(l)) {
-    if (NNUMBERP(tmp=EVALCAR(l))) goto Error;
+    tmp = from_eval? EVALCAR(l): CAR(l);
+    if (NNUMBERP(tmp)) goto Error;
     inexact |= FLONUMP(tmp);
     if (do_compare(tmp, max) > 0) max = tmp;
   }
-  return (inexact && EXACTP(max)) ? exact2inexact(max) : max;
+  return (inexact && EXACTP(max)) ? STk_exact2inexact(max) : max;
 Error:
-  err("max: bad number", tmp);
+  Err("max: bad number", tmp);
+  return UNDEFINED; /* never reached */
 }
 
-PRIMITIVE lmin(SCM l, SCM env)
+PRIMITIVE STk_min(SCM l, SCM env, int from_eval)
 {
   register SCM tmp, min;
   int inexact;
 
-  if (NULLP(l)) err("min: bad number of arguments", NIL);
-  
-  if (NNUMBERP(tmp=EVALCAR(l))) goto Error;
+  if (NULLP(l)) Err("min: bad number of arguments", NIL);
+
+  tmp = from_eval? EVALCAR(l): CAR(l);
+  if (NNUMBERP(tmp)) goto Error;
   inexact = FLONUMP(tmp);
   min = tmp;
   
   for (l=CDR(l); NNULLP(l); l=CDR(l)) {
-    if (NNUMBERP(tmp=EVALCAR(l))) goto Error;
+    tmp = from_eval? EVALCAR(l): CAR(l);
+    if (NNUMBERP(tmp)) goto Error;
     inexact |= FLONUMP(tmp);
     if (do_compare(tmp, min) < 0) min = tmp;
   }
-  return (inexact && EXACTP(min)) ? exact2inexact(min) : min;
+  return (inexact && EXACTP(min)) ? STk_exact2inexact(min) : min;
 Error:
-  err("min: bad number", tmp);
+  Err("min: bad number", tmp);
+  return UNDEFINED; /* never reached */
 }
 
-PRIMITIVE plus(SCM l, SCM env)
+PRIMITIVE STk_plus(SCM l, SCM env, int from_eval)
 {
   SCM tmp, res = makesmallint(0);
   
   for ( ; CONSP(l); l=CDR(l)) {
-    tmp = EVALCAR(l);
-    if (NNUMBERP(tmp)) err("+: not a number", tmp);
+    tmp = from_eval? EVALCAR(l): CAR(l);
+    if (NNUMBERP(tmp)) Err("+: not a number", tmp);
     do_addition(&res, tmp);
   }
   return res;
 }
 
-PRIMITIVE difference(SCM l, SCM env)
+PRIMITIVE STk_difference(SCM l, SCM env, int from_eval)
 {
   SCM tmp, res;
 
-  if (NULLP(l)) err("-: no argumenent given", NIL);
-  if (NULLP(CDR(l))) l = cons(makesmallint(0), l);	/* (- x) --> (- 0 x) */
+  if (NULLP(l)) Err("-: no argument given", NIL);
+  if (NULLP(CDR(l))) l = Cons(makesmallint(0), l);	/* (- x) --> (- 0 x) */
 
-  res = clone(EVALCAR(l));
+  tmp = res = clone(EVALCAR(l));
   if (NNUMBERP(res)) goto Error;
   for (l=CDR(l) ; CONSP(l); l=CDR(l)) {
-    tmp = EVALCAR(l);
+    tmp = from_eval? EVALCAR(l): CAR(l);
     if (NNUMBERP(tmp)) goto Error;
     do_substract(&res, tmp);
   }
   return res;
 Error:
-  err("-: not a number", tmp);
+  Err("-: not a number", tmp);
+  return UNDEFINED; /* never reached */
 }
 
-PRIMITIVE ltimes(SCM l, SCM env)
+PRIMITIVE STk_times(SCM l, SCM env, int from_eval)
 {
   SCM tmp, res = makesmallint(1);
 
   for ( ; CONSP(l); l=CDR(l)) {
-    tmp = EVALCAR(l);
-    if (NNUMBERP(tmp)) err("*: not a number", tmp);
+    tmp = from_eval? EVALCAR(l): CAR(l);
+    if (NNUMBERP(tmp)) Err("*: not a number", tmp);
     do_multiply(&res, tmp);
   }
   return res;
 }
 
-PRIMITIVE division(SCM l, SCM env)
+PRIMITIVE STk_division(SCM l, SCM env, int from_eval)
 {
   SCM tmp, res;
 
-  if (NULLP(l)) err("/: no argumenent given", NIL);
-  if (NULLP(CDR(l))) l = cons(makesmallint(1), l);	/* (/ x) --> (/ 1 x) */
+  if (NULLP(l)) Err("/: no argumenent given", NIL);
+  if (NULLP(CDR(l))) l = Cons(makesmallint(1), l);	/* (/ x) --> (/ 1 x) */
   
   tmp = res = clone(EVALCAR(l));
   if (NNUMBERP(res)) goto Error;
   for (l=CDR(l) ; CONSP(l); l=CDR(l)) {
-    tmp = EVALCAR(l);
-    if (NNUMBERP(tmp) || zerop(tmp) == truth) goto Error;
+    tmp = from_eval? EVALCAR(l): CAR(l);
+    if (NNUMBERP(tmp) || STk_zerop(tmp) == Truth) goto Error;
     do_divide(&res, tmp);
   }
   return res;
 Error:
-  err("/: not a valid number", tmp);
+  Err("/: not a valid number", tmp);
+  return UNDEFINED; /* never reached */
 }
 
-PRIMITIVE labsolute(SCM x)
+PRIMITIVE STk_absolute(SCM x)
 {
   switch (TYPE(x)) {
     case tc_integer: return (INTEGER(x) < 0) ? makesmallint(-INTEGER(x)) : x;
-    case tc_flonum:  return (FLONM(x) < 0.0) ? makenumber(-FLONM(x))    : x;
+    case tc_flonum:  return (FLONM(x) < 0.0) ? STk_makenumber(-FLONM(x)) : x;
     case tc_bignum:  if (mpz_cmp_ui(BIGNUM(x), 0L) < 0) {
      			SCM tmp = clone(x);
 			mpz_neg(BIGNUM(tmp), BIGNUM(x));
 			return tmp;
 		      }
       		      return x;
-    default:         err("abs: bad number", x);
+    default:         Err("abs: bad number", x);
+		     return UNDEFINED; /* never reached */
   }
 }  
 
-PRIMITIVE lquotient(SCM n1, SCM n2)
+PRIMITIVE STk_quotient(SCM n1, SCM n2)
 {
   if (NNUMBERP(n1) || NNUMBERP(n2))
-    err("quotient: bad arguments", cons(n1, n2));
-  if (zerop(n2) == truth) err("quotient: division by 0", NIL);
+    Err("quotient: bad arguments", Cons(n1, n2));
+  if (STk_zerop(n2) == Truth) Err("quotient: division by 0", NIL);
       
   return do_integer_division(n1, n2, TRUE);
 }
 
-PRIMITIVE lremainder(SCM n1, SCM n2)
+PRIMITIVE STk_remainder(SCM n1, SCM n2)
 {
   if (NNUMBERP(n1) || NNUMBERP(n2))
-    err("remainder: bad arguments", cons(n1, n2));
-  if (zerop(n2) == truth) err("remainder: division by 0", NIL);
+    Err("remainder: bad arguments", Cons(n1, n2));
+  if (STk_zerop(n2) == Truth) Err("remainder: division by 0", NIL);
 
   return do_integer_division(n1, n2, FALSE);
 }
 
-PRIMITIVE lmodulo(SCM n1, SCM n2)
+PRIMITIVE STk_modulo(SCM n1, SCM n2)
 {
   SCM z;
 
   if (NNUMBERP(n1) || NNUMBERP(n2))
-    err("modulo: bad arguments", cons(n1, n2));
-  if (zerop(n2) == truth) err("modulo: division by 0", NIL);
-  
+    Err("modulo: bad arguments", Cons(n1, n2));
+  if (STk_zerop(n2) == Truth) Err("modulo: division by 0", NIL);
+
   z = do_integer_division(n1, n2, FALSE);
-  if (negativep(n1) != negativep(n2))
+  if (STk_negativep(n1) != STk_negativep(n2) && STk_zerop(z) != Truth) 
+    /*kerch@parc.xerox.com*/
     do_addition(&z, n2);
   return z;
 }
 
 
-static SCM lgcd2(SCM n1, SCM n2)
+static SCM gcd2(SCM n1, SCM n2)
 {
-  return (zerop(n2) == truth) ? n1 : lgcd2(n2, lmodulo(n1, n2));
+  return (STk_zerop(n2) == Truth) ? n1 : gcd2(n2, STk_modulo(n1, n2));
 }
 
-PRIMITIVE lgcd(SCM l, SCM env)
+PRIMITIVE STk_gcd(SCM l, SCM env, int from_eval)
 {
   register SCM tmp, res;
 
   if (NULLP(l)) return makesmallint(0L);
   
-  if (NNUMBERP(tmp=EVALCAR(l))) goto Error;
-  res = labsolute(tmp);
+  tmp = EVALCAR(l);
+  if (NNUMBERP(tmp)) goto Error;
+  res = STk_absolute(tmp);
   
   for (l=CDR(l); NNULLP(l); l=CDR(l)) {
-    if (NNUMBERP(tmp=EVALCAR(l))) goto Error;
-    res = labsolute(lgcd2(res, tmp));
+    tmp = from_eval? EVALCAR(l): CAR(l);
+    if (NNUMBERP(tmp)) goto Error;
+    res = STk_absolute(gcd2(res, tmp));
   }
   return res;
 Error:
-  err("gcd: bad number", tmp);
+  Err("gcd: bad number", tmp);
+  return UNDEFINED; /* never reached */
 }
 
-PRIMITIVE llcm(SCM l, SCM env)
+PRIMITIVE STk_lcm(SCM l, SCM env, int from_eval)
 {
   SCM tmp, res, gcd;
 
   if (NULLP(l)) return makesmallint(1L);
-  
-  if (NNUMBERP(tmp=EVALCAR(l))) goto Error;
-  res = labsolute(clone(tmp));
+
+  tmp = EVALCAR(l);
+  if (NNUMBERP(tmp)) goto Error;
+  res = STk_absolute(clone(tmp));
   
   for (l=CDR(l); NNULLP(l); l=CDR(l)) {
-    if (NNUMBERP(tmp=EVALCAR(l))) goto Error;
-    if (zerop(tmp)==truth) return tmp;
-    gcd = lgcd2(res, tmp);
+    tmp=EVALCAR(l);
+    if (NNUMBERP(tmp)) goto Error;
+    if (STk_zerop(tmp)==Truth) return tmp;
+    gcd = gcd2(res, tmp);
     do_multiply(&res, tmp);
     do_divide(&res, gcd);
-    res =labsolute(res);
+    res = STk_absolute(res);
   }
   return res;
 Error:
-  err("lcm: bad number", tmp);
+  Err("lcm: bad number", tmp);
+  return UNDEFINED; /* never reached */
 }
 
-PRIMITIVE lfloor(SCM x)
+PRIMITIVE STk_floor(SCM x)
 {
-  if (NNUMBERP(x)) err("floor: bad number", x);
-  if (FLONUMP(x)) return makenumber(floor(FLONM(x)));
+  if (NNUMBERP(x)) Err("floor: bad number", x);
+  if (FLONUMP(x)) return STk_makenumber(floor(FLONM(x)));
   return x;
 }
 
-PRIMITIVE lceiling(SCM x)
+PRIMITIVE STk_ceiling(SCM x)
 {
-  if (NNUMBERP(x)) err("ceiling: bad number", x);
-  if (FLONUMP(x)) return makenumber(ceil(FLONM(x)));
+  if (NNUMBERP(x)) Err("ceiling: bad number", x);
+  if (FLONUMP(x)) return STk_makenumber(ceil(FLONM(x)));
   return x;
 }
 
-PRIMITIVE ltruncate(SCM x)
+PRIMITIVE STk_truncate(SCM x)
 {
-  if (NNUMBERP(x)) err("truncate: bad number", x);
+  if (NNUMBERP(x)) Err("truncate: bad number", x);
   if (FLONUMP(x)) {
     double d = FLONM(x);
-    return makenumber(d < 0.0 ? ceil(d) : floor(d));
+    return STk_makenumber(d < 0.0 ? ceil(d) : floor(d));
   }
   return x;
 }
-  
-PRIMITIVE lround(SCM x)
+
+PRIMITIVE STk_round(SCM x)
 {
-  if (NNUMBERP(x)) err("round: bad number", x);
-  if (FLONUMP(x)) return makenumber(rint(FLONM(x)));
+  if (NNUMBERP(x)) Err("round: bad number", x);
+  if (FLONUMP(x)) {
+    /*  
+     * R4RS states that round must returns the closest integer to x, rounding to
+     * even when x is halfway between two integers (round to even is for consistency
+     * with IEEE standard).
+     * This explains the (too much) complicate computation below
+     */
+    double v 	      = FLONM(x); 
+    double v_plus_0_5 = v + 0.5;
+    double res        = floor(v_plus_0_5);
+
+    return STk_makenumber(
+      (v_plus_0_5 == res && v_plus_0_5/2 != floor(v_plus_0_5/2)) ? res-1: res);
+  }
   return x;
 }
 
-#ifdef __STDC__
-#define transcendental(fct)					\
-    SCM l##fct(SCM z)						\
-    {								\
-      if (NNUMBERP(z)) err(#fct ": bad number", z);		\
-      else return makenumber(fct(FLONM(exact2inexact(z))));	\
-    }
+#if !defined(WIN32) && !defined(__STDC__)
+#  define concat(a,b) a/**/b
 #else
-#define concat(a,b) a/**/b
-#define transcendental(fct)					\
-    SCM concat(l,fct)(SCM z)					\
-    {								\
-      if (NNUMBERP(z)) err("fct : bad number", z);		\
-      else return makenumber(fct(FLONM(exact2inexact(z))));	\
-    }
+#  define concat(a,b) a##b
 #endif
 
+#define transcendental(fct)                                             \
+    PRIMITIVE concat(STk_,fct)(SCM z)                                   \
+    {                                                                   \
+      if (NUMBERP(z))                                                   \
+        return STk_makenumber(fct(FLONM(STk_exact2inexact(z))));        \
+      Err("transcendental function: bad number", z);                    \
+      return UNDEFINED /* never reached */;                             \
+    }
 
 transcendental(exp)
 transcendental(log)
@@ -993,22 +1094,22 @@ transcendental(tan)
 transcendental(asin)
 transcendental(acos)
 
-PRIMITIVE latan(SCM y, SCM x)
+PRIMITIVE STk_atan(SCM y, SCM x)
 {
-  if (NNUMBERP(y)) err("atan: bad  number", y);
+  if (NNUMBERP(y)) Err("atan: bad  number", y);
 
   if (x == UNBOUND)
-    return makenumber(atan(FLONM(exact2inexact(y))));
+    return STk_makenumber(atan(FLONM(STk_exact2inexact(y))));
   else {
-    double fl_x = FLONM(exact2inexact(x));
-    double fl_y = FLONM(exact2inexact(y));
+    double fl_x = FLONM(STk_exact2inexact(x));
+    double fl_y = FLONM(STk_exact2inexact(y));
 
     /* Make a special case for (atan 0 0) -- in this case return 0 */
-    return makenumber((fl_x == 0 && fl_y == 0) ? 0L : atan2(fl_y, fl_x));
+    return STk_makenumber((fl_x == 0 && fl_y == 0) ? 0L : atan2(fl_y, fl_x));
   }
 }
 
-PRIMITIVE lsqrt(SCM z)
+PRIMITIVE STk_sqrt(SCM z)
 {
   switch (TYPE(z)) {
     case tc_integer: {
@@ -1016,7 +1117,7 @@ PRIMITIVE lsqrt(SCM z)
 	 
 	 if (INTEGER(z) < 0) goto Error;
 	 d = (double) sqrt((double) INTEGER(z));
-	 return ISINT(d) ? makesmallint((long) d): makenumber(d);
+	 return ISINT(d) ? makesmallint((long) d): STk_makenumber(d);
        }
     case tc_bignum: {
          MP_INT root, remainder;
@@ -1032,20 +1133,21 @@ PRIMITIVE lsqrt(SCM z)
 	 }
 	 else {
 	   /* Result is a flonum */
-	   res = makenumber(bignum2double(&root));
+	   res = STk_makenumber(bignum2double(&root));
 	 }
 	 mpz_clear(&root); mpz_clear(&remainder);
 	 return res;
        }
     case tc_flonum: {
          if (FLONM(z) < 0.0) goto Error;
-	 return makenumber((double) sqrt(FLONM(z)));
+	 return STk_makenumber((double) sqrt(FLONM(z)));
        }
     default:
-         err("sqrt: bad number", z);
+         Err("sqrt: bad number", z);
   }
 Error:
-  err("sqrt: number is negative", z);
+  Err("sqrt: number is negative", z);
+  return UNDEFINED; /* never reached */
 }
 
 /*
@@ -1061,14 +1163,14 @@ static SCM exact_expt(SCM z1, SCM z2)
 {
   SCM res;
   
-  if (zerop(z2) == truth) return makesmallint(1);
-  if (negativep(z2) == truth) { 
+  if (STk_zerop(z2) == Truth) return makesmallint(1);
+  if (STk_negativep(z2) == Truth) { 
     /* (/ 1 (expt z1 (abs z2) */
     res = makesmallint(1);
-    do_divide(&res, exact_expt(z1, labsolute(z2)));
+    do_divide(&res, exact_expt(z1, STk_absolute(z2)));
     return res;
   }
-  if (evenp(z2) == truth) {
+  if (STk_evenp(z2) == Truth) {
     /* (square (expt z1 (/ z2 2)) */
     res = clone(z2);
     do_divide(&res, makesmallint(2));
@@ -1084,53 +1186,60 @@ static SCM exact_expt(SCM z1, SCM z2)
   return res;
 }
 
-PRIMITIVE lexpt(SCM z1, SCM z2)
+PRIMITIVE STk_expt(SCM z1, SCM z2)
 {
-  if (NNUMBERP(z1)) err("expt: bad number", z1);
-  if (NNUMBERP(z2)) err("expt: bad number", z2);
+  if (NNUMBERP(z1)) Err("expt: bad number", z1);
+  if (NNUMBERP(z2)) Err("expt: bad number", z2);
 
   return (EXACTP(z1) && EXACTP(z2)) ?
     		exact_expt(z1, z2)  :
-		makenumber((double) pow(FLONM(exact2inexact(z1)),
-					FLONM(exact2inexact(z2))));
+		STk_makenumber((double) pow(FLONM(STk_exact2inexact(z1)),
+					    FLONM(STk_exact2inexact(z2))));
 }
 
-PRIMITIVE exact2inexact(SCM z)
+PRIMITIVE STk_exact2inexact(SCM z)
 {
   switch (TYPE(z)) {
-    case tc_integer: return makenumber((double) INTEGER(z));
-    case tc_bignum:  return makenumber(bignum2double(BIGNUM(z)));
+    case tc_integer: return STk_makenumber((double) INTEGER(z));
+    case tc_bignum:  return STk_makenumber(bignum2double(BIGNUM(z)));
     case tc_flonum:  return z;
-    default:         err("exact->inexact: bad number", z);
+    default:         Err("exact->inexact: bad number", z);
+		     return UNDEFINED; /* never reached */
   }
-  return z;
 }
 
-PRIMITIVE inexact2exact(SCM z)
+PRIMITIVE STk_inexact2exact(SCM z)
 {
   switch (TYPE(z)) {
     case tc_integer: 
     case tc_bignum:  return z;
-    case tc_flonum:  return double2bignum(floor(FLONM(z)+0.5));
-    default:         err("inexact->exact: bad number", z);
+    case tc_flonum:  return double2integer(floor(FLONM(z)+0.5));
+    default:         Err("inexact->exact: bad number", z);
+      		     return UNDEFINED; /* never reached */
   }
 }
 
 
-PRIMITIVE string2number(SCM str, SCM base)
+PRIMITIVE STk_string2number(SCM str, SCM base)
 {
-  if (NSTRINGP(str)) err("string->number: not a string", str);
-  return Cstr2number(CHARS(str), (base == UNBOUND)? 10 : integer_value(base));
+  if (NSTRINGP(str)) Err("string->number: not a string", str);
+  return STk_Cstr2number(CHARS(str), (base==UNBOUND)? 10 : STk_integer_value(base));
 }
 
-PRIMITIVE number2string(SCM n, SCM base)
+PRIMITIVE STk_number2string(SCM n, SCM base)
 {
   char *s, buffer[100];
   SCM z;
 
-  if (NNUMBERP(n))   err("number->string: bad number", n);
-  s = number2Cstr(n, (base == UNBOUND)? 10 : integer_value(base), buffer);
-  z = makestrg(strlen(s), s);
+  if (NNUMBERP(n))   Err("number->string: bad number", n);
+  s = STk_number2Cstr(n, (base == UNBOUND)? 10 : STk_integer_value(base), buffer);
+  z = STk_makestring(s);
   if (s != buffer) free(s);
   return z;
 }
+
+PRIMITIVE  STk_bignump(SCM n)
+{
+  return BIGNUMP(n) ? Truth: Ntruth;
+}
+
