@@ -2,7 +2,7 @@
  *
  * p r i m i t i v e s . c			-- List of STk subrs
  *
- * Copyright (C) 1993,1994,1995 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright  1993-1996 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  * 
  *
  * Permission to use, copy, and/or distribute this software and its
@@ -19,7 +19,7 @@
  *
  *           Author: Erick Gallesio [eg@kaolin.unice.fr]
  *    Creation date: ??????
- * Last file update:  9-Jul-1995 11:58
+ * Last file update: 16-Jun-1996 22:54
  */
 
 
@@ -34,9 +34,6 @@ struct Primitive {
 #ifdef USE_HASH
 extern PRIMITIVE STk_init_hash(void);
 #endif
-#ifdef USE_STKLOS
-extern PRIMITIVE STk_init_stklos(void);
-#endif
 #ifdef USE_SOCKET
 extern PRIMITIVE STk_init_socket(void);
 #endif
@@ -52,6 +49,12 @@ extern PRIMITIVE STk_init_process(void);
 #ifdef USE_POSIX
 extern PRIMITIVE STk_init_posix(void);
 #endif
+#ifdef USE_HTML
+extern PRIMITIVE STk_init_html(void);
+#endif
+#ifdef USE_PIXMAP
+extern PRIMITIVE STk_init_pixmap(void);
+#endif
 
 static struct Primitive Scheme_primitives[] = { 
 
@@ -194,6 +197,7 @@ static struct Primitive Scheme_primitives[] = {
   {"inexact->exact",	    tc_subr_1,	    STk_inexact2exact},
   {"string->number",	    tc_subr_1_or_2, STk_string2number},
   {"number->string",	    tc_subr_1_or_2, STk_number2string},
+  {"bignum?",		    tc_subr_1,	    STk_bignump},		/* + */
 
   /**** Section 6.6 ****/
   {"char?",		    tc_subr_1,	    STk_charp},
@@ -313,6 +317,10 @@ static struct Primitive Scheme_primitives[] = {
   {"try-load",		    tc_subr_1,	    STk_try_load},		/* + */
   {"autoload",		    tc_fsubr,	    STk_autoload},		/* + */
   {"autoload?",		    tc_fsubr,	    STk_autoloadp},		/* + */
+#ifdef USE_TK
+  {"when-port-readable",    tc_subr_1_or_2, STk_when_port_readable},	/* + */
+  {"when-port-writable",    tc_subr_1_or_2, STk_when_port_writable},	/* + */
+#endif
   {"format",		    tc_lsubr,	    STk_format},		/* + */
   {"error",		    tc_lsubr,	    STk_error},			/* + */
   {"input-string-port?",    tc_subr_1,	    STk_input_string_portp},	/* + */
@@ -337,7 +345,6 @@ static struct Primitive Scheme_primitives[] = {
   {"string->widget",	    tc_subr_1,	    STk_string2widget},		/* + */
   {"tk-command?",	    tc_subr_1,	    STk_tk_commandp},		/* + */
   {"widget-name",	    tc_subr_1,	    STk_widget_name},		/* + */
-  {"widget-environment",    tc_subr_1,	    STk_widget_environment},	/* + */
   {"get-widget-data",	    tc_subr_1,	    STk_get_widget_data},	/* + */
   {"set-widget-data!",	    tc_subr_2,	    STk_set_widget_data},	/* + */
 #endif
@@ -351,6 +358,8 @@ static struct Primitive Scheme_primitives[] = {
   {"procedure-environment", tc_subr_1,	    STk_procedure_environment},	/* + */
   {"symbol-bound?",	    tc_subr_1_or_2, STk_symbol_boundp},		/* + */
   {"eval",		    tc_subr_1_or_2, STk_user_eval},		/* + */
+  {"eval-hook",		    tc_subr_3,	    STk_eval_hook},		/* + */
+
 
   /**** Section 6.14 ****/
   {"macro",		    tc_fsubr,	    STk_macro},			/* + */
@@ -364,12 +373,19 @@ static struct Primitive Scheme_primitives[] = {
   {"address?",		    tc_subr_1,	    STk_addressp},		/* + */
 
   /**** Section 6.16 ****/
+  {"set-signal-handler!",   tc_subr_2,	    STk_set_signal_handler},	/* + */
+  {"add-signal-handler!",   tc_subr_2, 	    STk_add_signal_handler},	/* + */
+  {"get-signal-handlers",   tc_subr_0_or_1, STk_get_signal_handlers},	/* + */
+
+  /**** Section 6.17 ****/
   {"getcwd",		    tc_subr_0,	    STk_getcwd},		/* + */
   {"chdir",		    tc_subr_1,	    STk_chdir},			/* + */
   {"getpid",		    tc_subr_0,	    STk_getpid},		/* + */
   {"expand-file-name",	    tc_subr_1,	    STk_expand_file_name},	/* + */
+  {"canonical-path",	    tc_subr_1,	    STk_canonical_path},	/* + */
   {"system",		    tc_subr_1,	    STk_system},		/* + */
   {"getenv",		    tc_subr_1,	    STk_getenv},		/* + */
+  {"setenv!",		    tc_subr_2,	    STk_setenv},		/* + */
   {"file-is-directory?",    tc_subr_1,	    STk_file_is_directoryp},	/* + */
   {"file-is-regular?",      tc_subr_1,	    STk_file_is_regularp},	/* + */
   {"file-is-readable?",     tc_subr_1,	    STk_file_is_readablep},	/* + */
@@ -403,12 +419,15 @@ static struct Primitive Scheme_primitives[] = {
   {"%get-eval-stack",	    tc_subr_0,	    STk_get_eval_stack},
   {"%get-environment-stack",tc_subr_0,	    STk_get_env_stack},
   {"%find-cells",	    tc_subr_1,	    STk_find_cells},
+  {"%library-location",	    tc_subr_0,	    STk_library_location},
+
+#ifdef USE_STKLOS
+  {"%init-stklos",	    tc_subr_0,	    STk_init_STklos},
+#endif
+
 #ifdef USE_HASH
   {"%init-hash",	    tc_subr_0,	    STk_init_hash},
 #endif
-#ifdef USE_STKLOS
-  {"%init-stklos",	    tc_subr_0,	    STk_init_stklos},
-#endif
 #ifdef USE_SOCKET
   {"%init-socket",	    tc_subr_0,	    STk_init_socket},
 #endif
@@ -424,12 +443,18 @@ static struct Primitive Scheme_primitives[] = {
 #ifdef USE_POSIX
  {"%init-posix",    	    tc_subr_0,	    STk_init_posix},
 #endif 
+#ifdef USE_HTML
+  {"%init-html",	    tc_subr_0,	    STk_init_html},
+#endif
+#if defined(USE_TK) && defined(USE_PIXMAP)
+  {"%init-pixmap",	    tc_subr_0,	    STk_init_pixmap},
+#endif
   { "", 0, (SCM (*)()) NULL }
 };
 
 void STk_init_primitives(void)
 {
-  register struct Primitive *p = Scheme_primitives;
+  register struct Primitive *p;
   register SCM z;
 
   for (p = Scheme_primitives; *p->name; p++) {
@@ -440,7 +465,3 @@ void STk_init_primitives(void)
     VCELL(Intern(p->name))  =  z;
   }
 }
-
-
-
-
