@@ -1,7 +1,7 @@
 /*
  * s l i b . c				-- Misc functions
  *
- * Copyright (C) 1993,1994,1995 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright  1993-1996 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  * 
  *
  * Permission to use, copy, and/or distribute this software and its
@@ -17,15 +17,31 @@
  *
  *           Author: Erick Gallesio [eg@unice.fr]
  *    Creation date: ??-Oct-1993 ??:?? 
- * Last file update: 16-Jul-1995 10:30
+ * Last file update: 25-Sep-1996 17:55
  *
  */
 
+#ifdef WIN32
+#  include <windows.h>
+#endif
+
 #include "stk.h"
 #include "gc.h"
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/times.h>
+
+#ifdef WIN32
+#   include <time.h>
+#   include <dos.h>
+#   include <process.h>
+#else
+#   include <stdarg.h>
+#   include <sys/times.h>
+#endif
+
+#ifdef USE_TK
+#   include "tk-glue.h"
+#endif
 
 
 
@@ -39,6 +55,22 @@
 #undef realloc
 #endif
 
+#define MAX_MALLOC_BEFORE_GC 1<<20	/* 1 Mb should suffice before calling GC */
+
+static unsigned long malloc_count = 0;
+
+
+static void cannot_allocate()
+{
+  fprintf(stderr, "**** Storage allocation from system failed\n");
+  fprintf(stderr, "**** Evaluation aborted\n");
+  
+  STk_gc_requested = 1;
+
+  /* Use a null message to avoid a call to report-error (which uses conses) */
+  Err("", NIL);
+}
+
 void *STk_must_malloc(unsigned long size)
 {
   void *tmp;
@@ -46,9 +78,20 @@ void *STk_must_malloc(unsigned long size)
   tmp = (void *) malloc(size);
 
   /* Test for size because some libc return NULL when doing malloc(0) */
-  if (tmp == NULL && size)
-    Err("failed to allocate storage from system", NIL);
-  return(tmp);
+  if (tmp == NULL && size) cannot_allocate();
+
+  /* Idea of malloc limitation comes from Harvey J. Stein 
+   * <hjstein@MATH.HUJI.AC.IL>. The following code provoke a GC when
+   * MAX_MALLOC_BEFORE_GC have been allocated by must_malloc. This 
+   * allows to GC before all cells have been exhausted 
+   */
+  malloc_count +=size;
+  if (malloc_count > MAX_MALLOC_BEFORE_GC) {
+    malloc_count     = 0;
+    STk_gc_requested = 1;
+  }
+
+  return tmp;
 }
 
 void *STk_must_realloc(void *ptr, unsigned long size)
@@ -56,20 +99,22 @@ void *STk_must_realloc(void *ptr, unsigned long size)
   void *tmp;
 
   tmp = (void *) realloc(ptr, size);
-  if (tmp == NULL)
-    Err("failed to re-allocate storage from system",NIL);
-  return(tmp);
+  /* Since we cannot know (in a portable way) the size of area pointed by ptr,
+   * we will make the assumption that it is half the new requested size. 
+   * Of course, we are probably false here, but it seems more reasonable than 
+   * brutally increment it with size. 
+   */
+  malloc_count +=size/2;
+  if (malloc_count > MAX_MALLOC_BEFORE_GC) {
+    malloc_count     = 0;
+    STk_gc_requested = 1;
+  }
+
+  if (tmp == NULL) cannot_allocate();
+  return tmp;
 }
 #endif
 
-int STk_strcmpi(register char *p1, register char *p2)
-{
-  for( ; tolower(*p1) == tolower(*p2); p1++, p2++) 
-    if (!*p1) return 0;
-
-  return tolower(*p1) - tolower(*p2);
-}
-
 
 SCM STk_internal_eval_string(char *s, long context, SCM env)
 {
@@ -86,7 +131,7 @@ SCM STk_internal_eval_string(char *s, long context, SCM env)
   if ((k = setjmp(jb)) == 0) {
     Top_jmp_buf   = &jb;
     Error_context = (Error_context & ERR_IGNORED) | context;
-    result = STk_eval(STk_readf(port->storage_as.port.f, FALSE), env);
+    result = STk_eval(STk_readf(PORT_FILE(port), FALSE), env);
   }
   Top_jmp_buf   = prev_jb;;
   Error_context = prev_context;
@@ -136,8 +181,15 @@ PRIMITIVE STk_quit_interpreter(SCM retcode)
 
   /* call user finalization code */
   STk_user_cleanup();
+
 #ifdef USE_TK
-  Tcl_DeleteInterp(STk_main_interp); /* Unregister the interpreter from X server */
+  /* Unregister the interpreter from X server */
+  if (Tk_initialized) Tcl_DeleteCommand(STk_main_interp, ".");
+#endif
+
+#if defined(WIN32) && defined(USE_SOCKET)
+  /* Unregister the interpreter from Winsock */
+  WSACleanup();  
 #endif
   exit(ret);
 }
@@ -152,6 +204,11 @@ PRIMITIVE STk_machine_type(void)
   return STk_makestring(MACHINE);
 }
 
+PRIMITIVE STk_library_location(void)
+{
+   return STk_makestring(STk_library_path);
+}
+
 PRIMITIVE STk_random(SCM n)
 {
   if (NEXACTP(n) || STk_negativep(n) == Truth || STk_zerop(n) == Truth)
@@ -170,11 +227,21 @@ PRIMITIVE STk_set_random_seed(SCM n)
 #define HZ 60.0
 #endif
 
+#ifdef CLOCKS_PER_SEC
+#  define TIC CLOCKS_PER_SEC
+#else 
+#  define TIC HZ
+#endif
+
 double STk_my_time(void)
 {
+#ifdef WIN32
+  return (long) 1000*(clock()/CLK_TCK);
+#else
   struct tms time_buffer;
   times(&time_buffer);
-  return 1000 * (time_buffer.tms_utime + time_buffer.tms_stime) / HZ;
+  return 1000 * (time_buffer.tms_utime + time_buffer.tms_stime) / TIC;
+#endif
 }
 
 
@@ -223,12 +290,53 @@ PRIMITIVE STk_time(SCM expr, SCM env, int len)
   gc_time         = STk_total_gc_time;
   rt 	          = STk_my_time();
   res 	          = EVALCAR(expr);
-  fprintf(stderr, ";;    Time: %.2fms\n;; GC time: %.2fms\n;;   Cells: %ld\n",
+  fprintf(STk_stderr, ";;    Time: %.2fms\n;; GC time: %.2fms\n;;   Cells: %ld\n",
 	  STk_my_time()-rt, STk_total_gc_time-gc_time, STk_alloc_cells);
   return res;
 }
 
 
+/*
+ * STk_delete_Tcl_child_Interp
+ *
+ * 	This function must be called by a child process just after a fork
+ * It deletes the Tk commands associated to the interpreter (except send, 
+ * to avoid interpreter unregistering).
+ * 
+ * This must probably rewritten
+ */
+
+void STk_delete_Tcl_child_Interp(void)
+{
+#ifdef USE_TK
+  if (Tk_initialized) {
+    struct Tk_command *W;
+    Interp *iPtr = (Interp *) STk_main_interp;
+    Tcl_HashEntry *hPtr;
+
+    /* Try to find "send". Modify it's delproc to point NULL */
+    hPtr = Tcl_FindHashEntry(&iPtr->commandTable, "send");
+    if (hPtr != NULL) {
+      W = (struct Tk_command *) Tcl_GetHashValue(hPtr);
+      W->delproc = NULL;
+    }
+    /* Now we can destroy the interpreter  (send will not be destroyed) */
+    Tcl_DeleteInterp(STk_main_interp);
+    
+    /* Report-error points to a graphical procedure. Undefine it 
+     * to display error messages on stderr in the child process
+     */
+    STk_set_symbol_value("report-error", UNBOUND);
+
+    /* Redefine exit to the standard STk exit function */
+    STk_add_new_primitive("exit", tc_subr_0_or_1, STk_quit_interpreter);
+  }
+#else
+  /* Nothing to do for Snow */
+#endif
+}
+
+
 /* When STk evaluates an expression, it recodes it in a manner which permits it
    to be more efficient for further evaluations. The uncode functions permits to 
    do the reverse job: it takes an exppression and returns a form similar to the 
@@ -299,56 +407,29 @@ PRIMITIVE STk_uncode(SCM expr)
   }
 }
 
-
-#ifdef USE_TK
-/******************************************************************************
- *
- * Trace var
- *
- ******************************************************************************/
-
-static char *TraceVarFct(ClientData clientData, Tcl_Interp *interp, 
-			 char *name1, char *name2,
-			 int flags)
+/* 
+ * A Panic procedure.
+ */
+void STk_panic TCL_VARARGS_DEF(char *,arg1)
 {
-  /* 
-   * ClientData is the only field which insterest us here. It contains a 
-   * string to evaluate 
-   */
-  Tcl_GlobalEval(interp, (char*) clientData);
-  return NULL; /* to make the compiler happy */
-}
+  va_list argList;
+  char buf[1024];
+  char *format;
+	 
+  format = TCL_VARARGS_START(char *,arg1,argList);
+  vsprintf(buf, format, argList);
 
-PRIMITIVE STk_trace_var(SCM var, SCM code)
-{
-  char *s1, *s2;
-  SCM dumb;
-  
-  if (NSYMBOLP(var)) Err("trace-var: bad variable name", var);
-
-  s1 = STk_convert_for_tk(code, &dumb);
-  /* 
-   * Build a copy of s1 and set it as the ClientData for Tcl_TraceVar
-   * Duplication of s1 permits to be sure that the hash table has its 
-   * own copy of the code to evaluate since original code can be garbaged.
-   * TCL_TRACE_READS flags is used here to indicate to untrace to free
-   * the code string.
-   */
-  s2 = (char *) must_malloc(strlen(s1) + 1); 
-  strcpy(s2, s1);
-
-  Tcl_TraceVar(STk_main_interp, PNAME(var), TCL_TRACE_READS|TCL_TRACE_WRITES, 
-	       TraceVarFct, (ClientData) s2);
-  return UNDEFINED;
-}
-
-PRIMITIVE STk_untrace_var(SCM var)
-{
-  if (NSYMBOLP(var)) Err("untrace-var: bad variable name", var);
-  Tcl_CompleteUntraceVar(STk_main_interp, PNAME(var));
-  return UNDEFINED;
-}
+#ifdef WIN32
+  MessageBeep(MB_ICONEXCLAMATION);
+  MessageBox(NULL, buf, "Fatal error in STk", 
+	     MB_ICONSTOP | MB_OK | MB_TASKMODAL | MB_SETFOREGROUND);
+#else
+  fprintf(STk_stderr, "\n**** %s\n", buf);
+  fflush(STk_stderr);
 #endif
+  exit(1);
+}
+
 
 /******************************************************************************
  *
@@ -357,11 +438,11 @@ PRIMITIVE STk_untrace_var(SCM var)
  * not find them and report an error
  *
  ******************************************************************************/
-
+#ifndef WIN32
 typedef void (*dumb)();
 
 dumb STk_dumb[] = { 
   (dumb) Tcl_TildeSubst,
   (dumb) Tcl_SetVar2
 };
-
+#endif
