@@ -3,7 +3,7 @@
  * t k - u t i l . c 		- Some Tk utilities 
  *
  *
- * Copyright (C) 1993,1994,1995 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
+ * Copyright  1993-1996 Erick Gallesio - I3S-CNRS/ESSI <eg@unice.fr>
  * 
  *
  * Permission to use, copy, and/or distribute this software and its
@@ -19,7 +19,7 @@
  *
  *            Author: Erick Gallesio [eg@unice.fr]
  *    Creation date: 19-Feb-1993 22:15
- * Last file update:  2-Feb-1995 22:40
+ * Last file update: 26-Apr-1996 23:17
  *
  */
 
@@ -80,34 +80,4 @@ PRIMITIVE STk_set_widget_data(SCM widget, SCM value)
   widget->storage_as.tk.l_data = value;
   return UNDEFINED;
 }
-
-PRIMITIVE STk_widget_environment(SCM widget)
-{
-  if (NTKCOMMP(widget)) Err("widget-environment: bad widget", widget);
-  return widget->storage_as.tk.data->environment;
-}
-
-
-/*
- * STk_Stringify permits to transform the string "s" in a valid STk string.
- * Original string is deallocated if free_original is 1 
- */
-
-char *STk_Stringify(char *s, int free_original)
-{
-  char *res, *d;
-  
-  if (s == NULL) s = "";
-  res = d = must_malloc(2 * strlen(s) + 3); /* worst overestimation */
-  
-  for ( *d++ = '"'; *s; s++, d++) {
-    if (*s == '"' || *s == '\\') *d++ = '\\';
-    *d = *s;
-  }
-  *d++ = '"';
-  *d   = '\0';
-  
-  if (free_original) free(s);
-  return res;
-}
 #endif /* USE_TK */
