@@ -4,12 +4,12 @@
  *	Declarations of things used internally by the Tcl interpreter.
  *
  * Copyright (c) 1987-1993 The Regents of the University of California.
- * Copyright (c) 1994 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1996 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * @(#) tclInt.h 1.100 94/12/17 16:14:51
+ * SCCS: @(#) tclInt.h 1.200 96/04/11 17:24:12
  */
 
 #ifndef _TCLINT
@@ -50,17 +50,11 @@
 #else
 #include <string.h>
 #endif
-#include <varargs.h>
-
-/*
- * At present (12/91) not all stdlib.h implementations declare strtod.
- * The declaration below is here to ensure that it's declared, so that
- * the compiler won't take the default approach of assuming it returns
- * an int.  There's no ANSI prototype for it because there would end
- * up being too many conflicts with slightly-different prototypes.
- */
-
-extern double strtod();
+#if defined(__STDC__) || defined(HAS_STDARG)
+#   include <stdarg.h>
+#else
+#   include <varargs.h>
+#endif
 
 /*
  *----------------------------------------------------------------
@@ -268,17 +262,16 @@ typedef struct Trace {
 } Trace;
 
 /*
- * The stucture below defines a deletion callback, which is
- * a procedure to invoke just before an interpreter is deleted.
+ * The structure below defines an entry in the assocData hash table which
+ * is associated with an interpreter. The entry contains a pointer to a
+ * function to call when the interpreter is deleted, and a pointer to
+ * a user-defined piece of data.
  */
 
-typedef struct DeleteCallback {
-    Tcl_InterpDeleteProc *proc;	/* Procedure to call. */
-    ClientData clientData;	/* Value to pass to procedure. */
-    struct DeleteCallback *nextPtr;
-				/* Next in list of callbacks for this
-				 * interpreter (or NULL for end of list). */
-} DeleteCallback;
+typedef struct AssocData {
+    Tcl_InterpDeleteProc *proc;	/* Proc to call when deleting. */
+    ClientData clientData;	/* Value to pass to proc. */
+} AssocData;    
 
 /*
  * The structure below defines a frame, which is a procedure invocation.
@@ -342,45 +335,11 @@ typedef struct HistoryRev {
 				 * current history event. */
     int newSize;		/* Number of bytes in newBytes. */
     char *newBytes;		/* Replacement for the range given by
-				 * firstIndex and lastIndex. */
+				 * firstIndex and lastIndex (malloced). */
     struct HistoryRev *nextPtr;	/* Next in chain of revisions to apply, or
 				 * NULL for end of list. */
 } HistoryRev;
 
-/*
- *----------------------------------------------------------------
- * Data structures related to files.  These are used primarily in
- * tclUnixUtil.c and tclUnixAZ.c.
- *----------------------------------------------------------------
- */
-
-/*
- * The data structure below defines an open file (or connection to
- * a process pipeline) as returned by the "open" command.
- */
-
-typedef struct OpenFile {
-    FILE *f;			/* Stdio file to use for reading and/or
-				 * writing. */
-    FILE *f2;			/* Normally NULL.  In the special case of
-				 * a command pipeline with pipes for both
-				 * input and output, this is a stdio file
-				 * to use for writing to the pipeline. */
-    int permissions;		/* OR-ed combination of TCL_FILE_READABLE
-				 * and TCL_FILE_WRITABLE. */
-    int numPids;		/* If this is a connection to a process
-				 * pipeline, gives number of processes
-				 * in pidPtr array below;  otherwise it
-				 * is 0. */
-    int *pidPtr;		/* Pointer to malloc-ed array of child
-				 * process ids (numPids of them), or NULL
-				 * if this isn't a connection to a process
-				 * pipeline. */
-    int errorId;		/* File id of file that receives error
-				 * output from pipeline.  -1 means not
-				 * used (i.e. this is a normal file). */
-} OpenFile;
-
 /*
  *----------------------------------------------------------------
  * Data structures related to expressions.  These are used only in
@@ -405,15 +364,21 @@ typedef struct MathFunc {
 
 /*
  *----------------------------------------------------------------
- * This structure defines an interpreter, which is a collection of
- * commands plus other state information related to interpreting
- * commands, such as variable storage.  Primary responsibility for
- * this data structure is in tclBasic.c, but almost every Tcl
- * source file uses something in here.
+ * One of the following structures exists for each command in
+ * an interpreter.  The Tcl_Command opaque type actually refers
+ * to these structures.
  *----------------------------------------------------------------
  */
 
 typedef struct Command {
+    Tcl_HashEntry *hPtr;	/* Pointer to the hash table entry in
+				 * interp->commandTable that refers to
+				 * this command.  Used to get a command's
+				 * name from its Tcl_Command handle.  NULL
+				 * means that the hash table entry has
+				 * been removed already (this can happen
+				 * if deleteProc causes the command to be
+				 * deleted or recreated). */
     Tcl_CmdProc *proc;		/* Procedure to process command. */
     ClientData clientData;	/* Arbitrary value to pass to proc. */
     Tcl_CmdDeleteProc *deleteProc;
@@ -421,9 +386,21 @@ typedef struct Command {
 				 * command. */
     ClientData deleteData;	/* Arbitrary value to pass to deleteProc
 				 * (usually the same as clientData). */
+    int deleted;		/* Means that the command is in the process
+				 * of being deleted (its deleteProc is
+				 * currently executing).  Any other attempts
+				 * to delete the command should be ignored. */
 } Command;
 
-#define CMD_SIZE(nameLength) ((unsigned) sizeof(Command) + nameLength - 3)
+/*
+ *----------------------------------------------------------------
+ * This structure defines an interpreter, which is a collection of
+ * commands plus other state information related to interpreting
+ * commands, such as variable storage.  Primary responsibility for
+ * this data structure is in tclBasic.c, but almost every Tcl
+ * source file uses something in here.
+ *----------------------------------------------------------------
+ */
 
 typedef struct Interp {
 
@@ -436,7 +413,9 @@ typedef struct Interp {
     char *result;		/* Points to result returned by last
 				 * command. */
     Tcl_FreeProc *freeProc;	/* Zero means result is statically allocated.
-				 * If non-zero, gives address of procedure
+				 * TCL_DYNAMIC means result was allocated with
+				 * ckalloc and should be freed with ckfree.
+				 * Other values give address of procedure
 				 * to invoke to free the result.  Must be
 				 * freed by Tcl_Eval before executing next
 				 * command. */
@@ -540,6 +519,19 @@ typedef struct Interp {
 				/* Compiled forms of above strings.  Also
 				 * malloc-ed, or NULL if not in use yet. */
 
+    /*
+     * Information about packages.  Used only in tclPkg.c.
+     */
+
+    Tcl_HashTable packageTable;	/* Describes all of the packages loaded
+				 * in or available to this interpreter.
+				 * Keys are package names, values are
+				 * (Package *) pointers. */
+    char *packageUnknown;	/* Command to invoke during "package
+				 * require" commands for packages that
+				 * aren't described in packageTable. 
+				 * Malloc'ed, may be NULL. */
+
     /*
      * Information used by Tcl_PrintDouble:
      */
@@ -573,9 +565,9 @@ typedef struct Interp {
 				 * to Tcl_EvalFile. */
     int flags;			/* Various flag bits.  See below. */
     Trace *tracePtr;		/* List of traces for this interpreter. */
-    DeleteCallback *deleteCallbackPtr;
-				/* First in list of callbacks to invoke when
-				 * interpreter is deleted. */
+    Tcl_HashTable *assocData;	/* Hash table for associating data with
+                                 * this interpreter. Cleaned up when
+                                 * this interpreter is deleted. */
     char resultSpace[TCL_RESULT_SIZE+1];
 				/* Static space for storing small results. */
 } Interp;
@@ -723,13 +715,51 @@ extern char tclTypeTable[];
 #define TCL_ALIGN(x) ((x + 7) & ~7)
 
 /*
- * Variables shared among Tcl modules but not used by the outside
- * world:
+ * For each event source (created with Tcl_CreateEventSource) there
+ * is a structure of the following type:
  */
 
-extern int		tclNumFiles;
-extern OpenFile **	tclOpenFiles;
-extern char *		tclRegexpError;
+typedef struct TclEventSource {
+    Tcl_EventSetupProc *setupProc;	/* This procedure is called by
+					 * Tcl_DoOneEvent to set up information
+					 * for the wait operation, such as
+					 * files to wait for or maximum
+					 * timeout. */
+    Tcl_EventCheckProc *checkProc;	/* This procedure is called by
+					 * Tcl_DoOneEvent after its wait
+					 * operation to see what events
+					 * are ready and queue them. */
+    ClientData clientData;		/* Arbitrary one-word argument to pass
+					 * to setupProc and checkProc. */
+    struct TclEventSource *nextPtr;	/* Next in list of all event sources
+					 * defined for applicaton. */
+} TclEventSource;
+
+/*
+ * The following macros are used to specify the runtime platform
+ * setting of the tclPlatform variable.
+ */
+
+typedef enum {
+    TCL_PLATFORM_UNIX,		/* Any Unix-like OS. */
+    TCL_PLATFORM_MAC,		/* MacOS. */
+    TCL_PLATFORM_WINDOWS	/* Any Microsoft Windows OS. */
+} TclPlatformType;
+
+/*
+ *----------------------------------------------------------------
+ * Variables shared among Tcl modules but not used by the outside
+ * world:
+ *----------------------------------------------------------------
+ */
+
+extern Tcl_Time			tclBlockTime;
+extern int			tclBlockTimeSet;
+extern char *			tclExecutableName;
+extern TclEventSource *		tclFirstEventSourcePtr;
+extern Tcl_ChannelType	 	tclFileChannelType;
+extern char *			tclMemDumpFileName;
+extern TclPlatformType		tclPlatform;
 
 /*
  *----------------------------------------------------------------
@@ -738,42 +768,136 @@ extern char *		tclRegexpError;
  *----------------------------------------------------------------
  */
 
-extern void		panic();
-extern void		TclCopyAndCollapse _ANSI_ARGS_((int count, char *src,
+EXTERN void		panic();
+EXTERN int		TclCleanupChildren _ANSI_ARGS_((Tcl_Interp *interp,
+		            int numPids, int *pidPtr, Tcl_Channel errorChan));
+EXTERN int		TclCloseFile _ANSI_ARGS_((Tcl_File file));
+EXTERN char *		TclConvertToNative _ANSI_ARGS_((Tcl_Interp *interp,
+			    char *name, Tcl_DString *bufferPtr));
+EXTERN char *		TclConvertToNetwork _ANSI_ARGS_((Tcl_Interp *interp,
+			    char *name, Tcl_DString *bufferPtr));
+EXTERN void		TclCopyAndCollapse _ANSI_ARGS_((int count, char *src,
 			    char *dst));
-extern void		TclDeleteVars _ANSI_ARGS_((Interp *iPtr,
+EXTERN int		TclChdir _ANSI_ARGS_((Tcl_Interp *interp,
+			    char *dirName));
+EXTERN void		TclClosePipeFile _ANSI_ARGS_((Tcl_File file));
+EXTERN Tcl_Channel	TclCreateCommandChannel _ANSI_ARGS_((
+    			    Tcl_File readFile, Tcl_File writeFile,
+			    Tcl_File errorFile, int numPids, int *pidPtr));
+EXTERN int              TclCreatePipe _ANSI_ARGS_((Tcl_File *readPipe,
+			    Tcl_File *writePipe));
+EXTERN int		TclCreatePipeline _ANSI_ARGS_((Tcl_Interp *interp,
+			    int argc, char **argv, int **pidArrayPtr,
+			    Tcl_File *inPipePtr,
+			    Tcl_File *outPipePtr,
+			    Tcl_File *errFilePtr));
+EXTERN Tcl_File		TclCreateTempFile _ANSI_ARGS_((char *contents));
+EXTERN void		TclDeleteVars _ANSI_ARGS_((Interp *iPtr,
 			    Tcl_HashTable *tablePtr));
-extern void		TclExpandParseValue _ANSI_ARGS_((ParseValue *pvPtr,
+EXTERN int		TclDoGlob _ANSI_ARGS_((Tcl_Interp *interp,
+			    char *separators, Tcl_DString *headPtr,
+			    char *tail));
+EXTERN void		TclExpandParseValue _ANSI_ARGS_((ParseValue *pvPtr,
 			    int needed));
-extern int		TclFindElement _ANSI_ARGS_((Tcl_Interp *interp,
+EXTERN void		TclExprFloatError _ANSI_ARGS_((Tcl_Interp *interp,
+			    double value));
+EXTERN int		TclFindElement _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *list, char **elementPtr, char **nextPtr,
 			    int *sizePtr, int *bracePtr));
-extern Proc *		TclFindProc _ANSI_ARGS_((Interp *iPtr,
+EXTERN Proc *		TclFindProc _ANSI_ARGS_((Interp *iPtr,
 			    char *procName));
-extern int		TclGetFrame _ANSI_ARGS_((Tcl_Interp *interp,
+EXTERN void		TclFreePackageInfo _ANSI_ARGS_((Interp *iPtr));
+EXTERN char *		TclGetCwd _ANSI_ARGS_((Tcl_Interp *interp));
+EXTERN unsigned long	TclGetClicks _ANSI_ARGS_((void));
+EXTERN char *		TclGetExtension _ANSI_ARGS_((char *name));
+EXTERN void		TclGetAndDetachPids _ANSI_ARGS_((Tcl_Interp *interp,
+		            Tcl_Channel chan));
+EXTERN int		TclGetDate _ANSI_ARGS_((char *p,
+			    unsigned long now, long zone,
+			    unsigned long *timePtr));
+EXTERN Tcl_Channel	TclGetDefaultStdChannel _ANSI_ARGS_((int type));
+EXTERN char *		TclGetEnv _ANSI_ARGS_((char *name));
+EXTERN int		TclGetFrame _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *string, CallFrame **framePtrPtr));
-extern int		TclGetListIndex _ANSI_ARGS_((Tcl_Interp *interp,
+EXTERN int		TclGetOpenMode _ANSI_ARGS_((Tcl_Interp *interp,
+        		    char *string, int *seekFlagPtr));
+EXTERN unsigned long	TclGetSeconds _ANSI_ARGS_((void));
+EXTERN void		TclGetTime _ANSI_ARGS_((Tcl_Time *time));
+EXTERN int		TclGetTimeZone _ANSI_ARGS_((unsigned long time));
+EXTERN char *		TclGetUserHome _ANSI_ARGS_((char *name,
+			    Tcl_DString *bufferPtr));
+EXTERN int		TclGetListIndex _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *string, int *indexPtr));
-extern Proc *		TclIsProc _ANSI_ARGS_((Command *cmdPtr));
-extern int		TclNeedSpace _ANSI_ARGS_((char *start, char *end));
-extern int		TclParseBraces _ANSI_ARGS_((Tcl_Interp *interp,
+EXTERN int		TclGetLoadedPackages _ANSI_ARGS_((Tcl_Interp *interp,
+			    char *targetName));
+EXTERN char *		TclGetUserHome _ANSI_ARGS_((char *name,
+			    Tcl_DString *bufferPtr));
+EXTERN int		TclGuessPackageName _ANSI_ARGS_((char *fileName,
+			    Tcl_DString *bufPtr));
+EXTERN int              TclHasPipes _ANSI_ARGS_((void));
+EXTERN int		TclHasSockets _ANSI_ARGS_((Tcl_Interp *interp));
+EXTERN int		TclIdlePending _ANSI_ARGS_((void));
+EXTERN int		TclInterpInit _ANSI_ARGS_((Tcl_Interp *interp));
+EXTERN Proc *		TclIsProc _ANSI_ARGS_((Command *cmdPtr));
+EXTERN int		TclLoadFile _ANSI_ARGS_((Tcl_Interp *interp,
+			    char *fileName, char *sym1, char *sym2,
+			    Tcl_PackageInitProc **proc1Ptr,
+			    Tcl_PackageInitProc **proc2Ptr));
+EXTERN int		TclMakeFileTable _ANSI_ARGS_((Tcl_Interp *interp,
+                            int noStdio));
+EXTERN int		TclMatchFiles _ANSI_ARGS_((Tcl_Interp *interp,
+			    char *separators, Tcl_DString *dirPtr,
+			    char *pattern, char *tail));
+EXTERN int		TclNeedSpace _ANSI_ARGS_((char *start, char *end));
+EXTERN Tcl_File		TclOpenFile _ANSI_ARGS_((char *fname, int mode));
+EXTERN int		TclParseBraces _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *string, char **termPtr, ParseValue *pvPtr));
-extern int		TclParseNestedCmd _ANSI_ARGS_((Tcl_Interp *interp,
+EXTERN int		TclParseNestedCmd _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *string, int flags, char **termPtr,
 			    ParseValue *pvPtr));
-extern int		TclParseQuotes _ANSI_ARGS_((Tcl_Interp *interp,
+EXTERN int		TclParseQuotes _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *string, int termChar, int flags,
 			    char **termPtr, ParseValue *pvPtr));
-extern int		TclParseWords _ANSI_ARGS_((Tcl_Interp *interp,
+EXTERN int		TclParseWords _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *string, int flags, int maxWords,
 			    char **termPtr, int *argcPtr, char **argv,
 			    ParseValue *pvPtr));
-extern char *		TclPrecTraceProc _ANSI_ARGS_((ClientData clientData,
+EXTERN void		TclPlatformExit _ANSI_ARGS_((int status));
+EXTERN void		TclPlatformInit _ANSI_ARGS_((Tcl_Interp *interp));
+EXTERN char *		TclPrecTraceProc _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, char *name1, char *name2,
 			    int flags));
-extern void		TclSetupEnv _ANSI_ARGS_((Tcl_Interp *interp));
-extern char *		TclWordEnd _ANSI_ARGS_((char *start, int nested,
+EXTERN int		TclPreventAliasLoop _ANSI_ARGS_((Tcl_Interp *interp,
+		            Tcl_Interp *cmdInterp, char *cmdName,
+                            Tcl_CmdProc *proc, ClientData clientData));
+EXTERN int		TclReadFile _ANSI_ARGS_((Tcl_File file,
+			    int shouldBlock, char *buf, int toRead));
+EXTERN int		TclSeekFile _ANSI_ARGS_((Tcl_File file,
+			    int offset, int whence));
+EXTERN int		TclServiceIdle _ANSI_ARGS_((void));
+EXTERN void		TclSetupEnv _ANSI_ARGS_((Tcl_Interp *interp));
+EXTERN int		TclSockGetPort _ANSI_ARGS_((Tcl_Interp *interp,
+		            char *string, char *proto, int *portPtr));
+EXTERN int		TclSockMinimumBuffers _ANSI_ARGS_((int sock,
+        		    int size));
+EXTERN int              TclSpawnPipeline _ANSI_ARGS_((Tcl_Interp *interp,
+	                    int *pidPtr, int *numPids, int argc, char **argv,
+			    Tcl_File inputFile,
+			    Tcl_File outputFile,
+	                    Tcl_File errorFile,
+	                    char *intIn, char *finalOut));
+EXTERN int		TclTestChannelCmd _ANSI_ARGS_((ClientData clientData,
+			    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int		TclTestChannelEventCmd _ANSI_ARGS_((
+    			    ClientData clientData, Tcl_Interp *interp,
+                            int argc, char **argv));
+EXTERN int		TclUpdateReturnInfo _ANSI_ARGS_((Interp *iPtr));
+EXTERN int		TclWaitForFile _ANSI_ARGS_((Tcl_File file,
+			    int mask, int timeout));
+EXTERN char *		TclWordEnd _ANSI_ARGS_((char *start, int nested,
 			    int *semiPtr));
+EXTERN int		TclWriteFile _ANSI_ARGS_((Tcl_File file,
+			    int shouldBlock, char *buf, int toWrite));
 
 /*
  *----------------------------------------------------------------
@@ -781,140 +905,171 @@ extern char *		TclWordEnd _ANSI_ARGS_((char *start, int nested,
  *----------------------------------------------------------------
  */
 
-extern int	Tcl_AppendCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_AfterCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_ArrayCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_AppendCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_BreakCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_ArrayCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_CaseCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_BreakCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_CatchCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_CaseCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_ConcatCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_CatchCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_ContinueCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_CdCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_ErrorCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_ClockCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_EvalCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_CloseCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_ExprCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_ConcatCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_ForCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_ContinueCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_ForeachCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int 	Tcl_CpCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_FormatCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int 	Tcl_EchoCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_GlobalCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_EofCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_HistoryCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_ErrorCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_IfCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_EvalCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_IncrCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_ExecCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_InfoCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_ExitCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_JoinCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_ExprCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_LappendCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_FblockedCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_LindexCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_FconfigureCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_LinsertCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_FileCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_LlengthCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_FileEventCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_ListCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_FlushCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_LrangeCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_ForCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_LreplaceCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_ForeachCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_LsearchCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_FormatCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_LsortCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_GetsCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_ProcCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_GlobalCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_RegexpCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_GlobCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_RegsubCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_HistoryCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_RenameCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_IfCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_ReturnCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_IncrCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_ScanCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_InfoCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_SetCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_InterpCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_SplitCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_JoinCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_StringCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_LappendCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_SubstCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_LindexCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_SwitchCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_LinsertCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_TraceCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_LlengthCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_UnsetCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_ListCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_UplevelCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_LoadCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_UpvarCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_LrangeCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_WhileCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_LreplaceCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_Cmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int 	Tcl_LsCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_Cmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_LsearchCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-
-/*
- *----------------------------------------------------------------
- * Command procedures in the UNIX core:
- *----------------------------------------------------------------
- */
-
-extern int	Tcl_CdCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_LsortCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_CloseCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int 	Tcl_MacBeepCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_EofCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int 	Tcl_MacSourceCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_ExecCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int 	Tcl_MkdirCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_ExitCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int 	Tcl_MvCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_FileCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_OpenCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_FlushCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_PackageCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_GetsCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_PidCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_GlobCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_ProcCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_OpenCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_PutsCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_PutsCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_PwdCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_PidCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_ReadCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_PwdCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_RegexpCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_ReadCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_RegsubCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_SeekCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_RenameCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_SourceCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int	Tcl_ReturnCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_TellCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int 	Tcl_RmCmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
-extern int	Tcl_TimeCmd _ANSI_ARGS_((ClientData clientData,
+EXTERN int 	Tcl_RmdirCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	Tcl_ScanCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	Tcl_SeekCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	Tcl_SetCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	Tcl_SplitCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	Tcl_SocketCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	Tcl_SourceCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	Tcl_StringCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	Tcl_SubstCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	Tcl_SwitchCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	Tcl_TellCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	Tcl_TimeCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	Tcl_TraceCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	Tcl_UnsetCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	Tcl_UpdateCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	Tcl_UplevelCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	Tcl_UpvarCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	Tcl_VwaitCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	Tcl_WhileCmd _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int	TclUnsupported0Cmd _ANSI_ARGS_((ClientData clientData,
 		    Tcl_Interp *interp, int argc, char **argv));
 
 #endif /* _TCLINT */
