@@ -5,17 +5,16 @@
  *	commands.
  *
  * Copyright (c) 1987-1993 The Regents of the University of California.
- * Copyright (c) 1994 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1995 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ *
+ * SCCS: @(#) tclUtil.c 1.112 96/02/15 11:42:52
  */
 
-#ifndef lint
-static char sccsid[] = "@(#) tclUtil.c 1.100 94/12/17 16:14:41";
-#endif
-
 #include "tclInt.h"
+#include "tclPort.h"
 
 /*
  * The following values are used in the flags returned by Tcl_ScanElement
@@ -81,7 +80,9 @@ static void		SetupAppendBuffer _ANSI_ARGS_((Interp *iPtr,
 
 int
 TclFindElement(interp, list, elementPtr, nextPtr, sizePtr, bracePtr)
-    Tcl_Interp *interp;		/* Interpreter to use for error reporting. */
+    Tcl_Interp *interp;		/* Interpreter to use for error reporting. 
+				 * If NULL, then no error message is left
+				 * after errors. */
     register char *list;	/* String containing Tcl list with zero
 				 * or more elements (possibly in braces). */
     char **elementPtr;		/* Fill in with location of first significant
@@ -116,7 +117,7 @@ TclFindElement(interp, list, elementPtr, nextPtr, sizePtr, bracePtr)
     if (*list == '(') {
 #else
     if (*list == '{') {
-#endif
+#endif    
 	openBraces = 1;
 	list++;
     } else if (*list == '"') {
@@ -142,9 +143,9 @@ TclFindElement(interp, list, elementPtr, nextPtr, sizePtr, bracePtr)
 	     */
 
 #ifdef STk_CODE
-	    case '(':
+ 	    case '(':
 #else
-	    case '{':
+  	    case '{':
 #endif
 		if (openBraces != 0) {
 		    openBraces++;
@@ -159,7 +160,7 @@ TclFindElement(interp, list, elementPtr, nextPtr, sizePtr, bracePtr)
 #ifdef STk_CODE
 	    case ')':
 #else
-	    case '}':
+  	    case '}':
 #endif
 		if (openBraces == 1) {
 		    char *p2;
@@ -173,10 +174,12 @@ TclFindElement(interp, list, elementPtr, nextPtr, sizePtr, bracePtr)
 			    && (p2 < p+20); p2++) {
 			/* null body */
 		    }
-		    Tcl_ResetResult(interp);
-		    sprintf(interp->result,
-			    "list element in braces followed by \"%.*s\" instead of space",
-			    (int) (p2-p), p);
+		    if (interp != NULL) {
+			Tcl_ResetResult(interp);
+			sprintf(interp->result,
+				"list element in braces followed by \"%.*s\" instead of space",
+				(int) (p2-p), p);
+		    }
 		    return TCL_ERROR;
 		} else if (openBraces != 0) {
 		    openBraces--;
@@ -230,10 +233,12 @@ TclFindElement(interp, list, elementPtr, nextPtr, sizePtr, bracePtr)
 			    && (p2 < p+20); p2++) {
 			/* null body */
 		    }
-		    Tcl_ResetResult(interp);
-		    sprintf(interp->result,
-			    "list element in quotes followed by \"%.*s\" %s",
-			    (int) (p2-p), p, "instead of space");
+		    if (interp != NULL) {
+			Tcl_ResetResult(interp);
+			sprintf(interp->result,
+				"list element in quotes followed by \"%.*s\" %s", (int) (p2-p), p,
+				"instead of space");
+		    }
 		    return TCL_ERROR;
 		}
 		break;
@@ -244,12 +249,16 @@ TclFindElement(interp, list, elementPtr, nextPtr, sizePtr, bracePtr)
 
 	    case 0:
 		if (openBraces != 0) {
-		    Tcl_SetResult(interp, "unmatched open brace in list",
-			    TCL_STATIC);
+		    if (interp != NULL) {
+			Tcl_SetResult(interp, "unmatched open brace in list",
+				TCL_STATIC);
+		    }
 		    return TCL_ERROR;
 		} else if (inQuotes) {
-		    Tcl_SetResult(interp, "unmatched open quote in list",
-			    TCL_STATIC);
+		    if (interp != NULL) {
+			Tcl_SetResult(interp, "unmatched open quote in list",
+				TCL_STATIC);
+		    }
 		    return TCL_ERROR;
 		}
 		size = p - list;
@@ -347,7 +356,8 @@ TclCopyAndCollapse(count, src, dst)
 
 int
 Tcl_SplitList(interp, list, argcPtr, argvPtr)
-    Tcl_Interp *interp;		/* Interpreter to use for error reporting. */
+    Tcl_Interp *interp;		/* Interpreter to use for error reporting. 
+				 * If NULL, then no error message is left. */
     char *list;			/* Pointer to string with list structure. */
     int *argcPtr;		/* Pointer to location to fill in with
 				 * the number of elements in the list. */
@@ -366,11 +376,31 @@ Tcl_SplitList(interp, list, argcPtr, argvPtr)
      * the number of space characters in the list.
      */
 
+#ifdef STk_CODE
+    {
+      char *q;
+
+      for (size = 1, p = q = list; *p != 0; p++) {
+	if (isspace(UCHAR(*p))) size++;
+	if (*p == ')') q = p; 
+      }
+
+      /* 
+       * Tcl considers strings of the form "( ... )" as quoted string
+       * (rather than lists). So if the string is of this form, open
+       * and close parenthesis are replaced by spaces
+       */
+      if (q > list && *list == '(' && *q == ')') {
+	*list = *q = ' ';
+      }
+    }
+#else
     for (size = 1, p = list; *p != 0; p++) {
 	if (isspace(UCHAR(*p))) {
 	    size++;
 	}
     }
+#endif
     size++;			/* Leave space for final NULL pointer. */
     argv = (char **) ckalloc((unsigned)
 	    ((size * sizeof(char *)) + (p - list) + 1));
@@ -386,13 +416,15 @@ Tcl_SplitList(interp, list, argcPtr, argvPtr)
 	}
 	if (i >= size) {
 	    ckfree((char *) argv);
-	    Tcl_SetResult(interp, "internal error in Tcl_SplitList",
-		    TCL_STATIC);
+	    if (interp != NULL) {
+		Tcl_SetResult(interp, "internal error in Tcl_SplitList",
+			TCL_STATIC);
+	    }
 	    return TCL_ERROR;
 	}
 	argv[i] = p;
 	if (brace) {
-	    strncpy(p, element, elSize);
+	    strncpy(p, element, (size_t) elSize);
 	    p += elSize;
 	    *p = 0;
 	    p++;
@@ -573,22 +605,20 @@ Tcl_ConvertElement(src, dst, flags)
      */
 #ifdef STk_CODE
     if ((src == NULL) || (*src == 0)) {
-	p[0] = '\\';
-	p[1] = '0';
-	p[2] = 0;
-	return 2;
+        p[0] = '\\';
+ 	p[1] = '0';
+ 	p[2] = 0;
+ 	return 2;
     }
     while (*p++ = *src++) /* Nothing */;
     return p - dst - 1;
 #else
     if ((src == NULL) || (*src == 0)) {
-        p[0] = '{';
+	p[0] = '{';
 	p[1] = '}';
 	p[2] = 0;
 	return 2;
     }
-
-      
     if ((flags & USE_BRACES) && !(flags & TCL_DONT_USE_BRACES)) {
 	*p = '{';
 	p++;
@@ -679,7 +709,6 @@ Tcl_ConvertElement(src, dst, flags)
     return p-dst;
 #endif
 }
-
 
 /*
  *----------------------------------------------------------------------
@@ -723,7 +752,11 @@ Tcl_Merge(argc, argv)
     } else {
 	flagPtr = (int *) ckalloc((unsigned) argc*sizeof(int));
     }
+#ifdef STk_CODE
+    numChars = 3; /* +2 cause of () */
+#else
     numChars = 1;
+#endif
     for (i = 0; i < argc; i++) {
 	numChars += Tcl_ScanElement(argv[i], &flagPtr[i]) + 1;
     }
@@ -733,18 +766,28 @@ Tcl_Merge(argc, argv)
      */
 
     result = (char *) ckalloc((unsigned) numChars);
+#ifdef STk_CODE
+    *result = '('; dst = result+1;
+#else
     dst = result;
+#endif
     for (i = 0; i < argc; i++) {
 	numChars = Tcl_ConvertElement(argv[i], dst, flagPtr[i]);
 	dst += numChars;
 	*dst = ' ';
 	dst++;
     }
+#ifdef STk_CODE
+    if (dst != result+1) dst -= 1;
+    dst[0] = ')';
+    dst[1] = '\0';
+#else
     if (dst == result) {
 	*dst = 0;
     } else {
 	dst[-1] = 0;
     }
+#endif
 
     if (flagPtr != localFlags) {
 	ckfree((char *) flagPtr);
@@ -810,7 +853,7 @@ Tcl_Concat(argc, argv)
 	if (length == 0) {
 	    continue;
 	}
-	(void) strncpy(p, element, length);
+	(void) strncpy(p, element, (size_t) length);
 	p += length;
 	*p = ' ';
 	p++;
@@ -992,16 +1035,18 @@ Tcl_SetResult(interp, string, freeProc)
     Tcl_FreeProc *oldFreeProc = iPtr->freeProc;
     char *oldResult = iPtr->result;
 
-    iPtr->freeProc = freeProc;
     if (string == NULL) {
 	iPtr->resultSpace[0] = 0;
 	iPtr->result = iPtr->resultSpace;
 	iPtr->freeProc = 0;
+    } else if (freeProc == TCL_DYNAMIC) {
+	iPtr->result = string;
+	iPtr->freeProc = TCL_DYNAMIC;
     } else if (freeProc == TCL_VOLATILE) {
 	length = strlen(string);
 	if (length > TCL_RESULT_SIZE) {
 	    iPtr->result = (char *) ckalloc((unsigned) length+1);
-	    iPtr->freeProc = (Tcl_FreeProc *) free;
+	    iPtr->freeProc = TCL_DYNAMIC;
 	} else {
 	    iPtr->result = iPtr->resultSpace;
 	    iPtr->freeProc = 0;
@@ -1009,6 +1054,7 @@ Tcl_SetResult(interp, string, freeProc)
 	strcpy(iPtr->result, string);
     } else {
 	iPtr->result = string;
+	iPtr->freeProc = freeProc;
     }
 
     /*
@@ -1018,7 +1064,8 @@ Tcl_SetResult(interp, string, freeProc)
      */
 
     if (oldFreeProc != 0) {
-	if (oldFreeProc == (Tcl_FreeProc *) free) {
+	if ((oldFreeProc == TCL_DYNAMIC)
+		|| (oldFreeProc == (Tcl_FreeProc *) free)) {
 	    ckfree(oldResult);
 	} else {
 	    (*oldFreeProc)(oldResult);
@@ -1026,31 +1073,6 @@ Tcl_SetResult(interp, string, freeProc)
     }
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * Tcl_Volatile --
- *
- *	This is a dummy procedure, in that it should never be
- *	invoked.  It is simply a placeholder.  It provides an
- *	address for TCL_VOLATILE to refer to, which won't be
- *	used by any other procedure.
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-void
-Tcl_Volatile(buffer)
-    char *buffer;			/* Not used. */
-{
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -1071,19 +1093,8 @@ Tcl_Volatile(buffer)
  */
 
 	/* VARARGS2 */
-#ifndef lint
 void
-Tcl_AppendResult(va_alist)
-#else
-void
-	/* VARARGS2 */ /* ARGSUSED */
-Tcl_AppendResult(interp, p, va_alist)
-    Tcl_Interp *interp;		/* Interpreter whose result is to be
-				 * extended. */
-    char *p;			/* One or more strings to add to the
-				 * result, terminated with NULL. */
-#endif
-    va_dcl
+Tcl_AppendResult TCL_VARARGS_DEF(Tcl_Interp *,arg1)
 {
     va_list argList;
     register Interp *iPtr;
@@ -1095,8 +1106,7 @@ Tcl_AppendResult(interp, p, va_alist)
      * needed.
      */
 
-    va_start(argList);
-    iPtr = va_arg(argList, Interp *);
+    iPtr = (Interp *) TCL_VARARGS_START(Tcl_Interp *,arg1,argList);
     newSpace = 0;
     while (1) {
 	string = va_arg(argList, char *);
@@ -1123,8 +1133,7 @@ Tcl_AppendResult(interp, p, va_alist)
      * them into the buffer.
      */
 
-    va_start(argList);
-    (void) va_arg(argList, Tcl_Interp *);
+    TCL_VARARGS_START(Tcl_Interp *,arg1,argList);
     while (1) {
 	string = va_arg(argList, char *);
 	if (string == NULL) {
@@ -1324,19 +1333,8 @@ Tcl_ResetResult(interp)
  *----------------------------------------------------------------------
  */
 	/* VARARGS2 */
-#ifndef lint
 void
-Tcl_SetErrorCode(va_alist)
-#else
-void
-	/* VARARGS2 */ /* ARGSUSED */
-Tcl_SetErrorCode(interp, p, va_alist)
-    Tcl_Interp *interp;		/* Interpreter whose errorCode variable is
-				 * to be set. */
-    char *p;			/* One or more elements to add to errorCode,
-				 * terminated with NULL. */
-#endif
-    va_dcl
+Tcl_SetErrorCode TCL_VARARGS_DEF(Tcl_Interp *,arg1)
 {
     va_list argList;
     char *string;
@@ -1348,8 +1346,7 @@ Tcl_SetErrorCode(interp, p, va_alist)
      * $errorCode as list elements.
      */
 
-    va_start(argList);
-    iPtr = va_arg(argList, Interp *);
+    iPtr = (Interp *) TCL_VARARGS_START(Tcl_Interp *,arg1,argList);
     flags = TCL_GLOBAL_ONLY | TCL_LIST_ELEMENT;
     while (1) {
 	string = va_arg(argList, char *);
@@ -1481,12 +1478,12 @@ Tcl_RegExpCompile(interp, string)
      * cache.
      */
 
-    tclRegexpError = NULL;
+    TclRegError((char *) NULL);
     result = TclRegComp(string);
-    if (tclRegexpError != NULL) {
+    if (TclGetRegError() != NULL) {
 	Tcl_AppendResult(interp,
 	    "couldn't compile regular expression pattern: ",
-	    tclRegexpError, (char *) NULL);
+	    TclGetRegError(), (char *) NULL);
 	return NULL;
     }
     if (iPtr->patterns[NUM_REGEXPS-1] != NULL) {
@@ -1517,7 +1514,8 @@ Tcl_RegExpCompile(interp, string)
  * Results:
  *	If an error occurs during the matching operation then -1
  *	is returned and interp->result contains an error message.
- *	Otherwise the return value is 1 if 
+ *	Otherwise the return value is 1 if a matching range is
+ *	found and 0 if there is no matching range.
  *
  * Side effects:
  *	None.
@@ -1539,12 +1537,12 @@ Tcl_RegExpExec(interp, re, string, start)
     int match;
 
     regexp *regexpPtr = (regexp *) re;
-    tclRegexpError = NULL;
+    TclRegError((char *) NULL);
     match = TclRegExec(regexpPtr, string, start);
-    if (tclRegexpError != NULL) {
+    if (TclGetRegError() != NULL) {
 	Tcl_ResetResult(interp);
 	Tcl_AppendResult(interp, "error while matching regular expression: ",
-		tclRegexpError, (char *) NULL);
+		TclGetRegError(), (char *) NULL);
 	return -1;
     }
     return match;
@@ -1702,7 +1700,8 @@ Tcl_DStringAppend(dsPtr, string, length)
     if (newSize >= dsPtr->spaceAvl) {
 	dsPtr->spaceAvl = newSize*2;
 	newString = (char *) ckalloc((unsigned) dsPtr->spaceAvl);
-	memcpy((VOID *)newString, (VOID *) dsPtr->string, dsPtr->length);
+	memcpy((VOID *)newString, (VOID *) dsPtr->string,
+		(size_t) dsPtr->length);
 	if (dsPtr->string != dsPtr->staticSpace) {
 	    ckfree(dsPtr->string);
 	}
@@ -1765,7 +1764,8 @@ Tcl_DStringAppendElement(dsPtr, string)
     if (newSize >= dsPtr->spaceAvl) {
 	dsPtr->spaceAvl = newSize*2;
 	newString = (char *) ckalloc((unsigned) dsPtr->spaceAvl);
-	memcpy((VOID *) newString, (VOID *) dsPtr->string, dsPtr->length);
+	memcpy((VOID *) newString, (VOID *) dsPtr->string,
+		(size_t) dsPtr->length);
 	if (dsPtr->string != dsPtr->staticSpace) {
 	    ckfree(dsPtr->string);
 	}
@@ -1828,7 +1828,8 @@ Tcl_DStringSetLength(dsPtr, length)
 	 * string in some cases.
 	 */
 
-	memcpy((VOID *) newString, (VOID *) dsPtr->string, dsPtr->length);
+	memcpy((VOID *) newString, (VOID *) dsPtr->string,
+		(size_t) dsPtr->length);
 	if (dsPtr->string != dsPtr->staticSpace) {
 	    ckfree(dsPtr->string);
 	}
@@ -1900,7 +1901,7 @@ Tcl_DStringResult(interp, dsPtr)
     Tcl_ResetResult(interp);
     if (dsPtr->string != dsPtr->staticSpace) {
 	interp->result = dsPtr->string;
-	interp->freeProc = (Tcl_FreeProc *) free;
+	interp->freeProc = TCL_DYNAMIC;
     } else if (dsPtr->length < TCL_RESULT_SIZE) {
 	interp->result = ((Interp *) interp)->resultSpace;
 	strcpy(interp->result, dsPtr->string);
@@ -1944,11 +1945,12 @@ Tcl_DStringGetResult(interp, dsPtr)
     }
     dsPtr->length = strlen(iPtr->result);
     if (iPtr->freeProc != NULL) {
-	if (iPtr->freeProc == (Tcl_FreeProc *) free) {
+	if ((iPtr->freeProc == TCL_DYNAMIC)
+		|| (iPtr->freeProc == (Tcl_FreeProc *) free)) {
 	    dsPtr->string = iPtr->result;
 	    dsPtr->spaceAvl = dsPtr->length+1;
 	} else {
-	    dsPtr->string = ckalloc((unsigned) (dsPtr->length+1));
+	    dsPtr->string = (char *) ckalloc((unsigned) (dsPtr->length+1));
 	    strcpy(dsPtr->string, iPtr->result);
 	    (*iPtr->freeProc)(iPtr->result);
 	}
@@ -1959,7 +1961,7 @@ Tcl_DStringGetResult(interp, dsPtr)
 	    dsPtr->string = dsPtr->staticSpace;
 	    dsPtr->spaceAvl = TCL_DSTRING_STATIC_SIZE;
 	} else {
-	    dsPtr->string = ckalloc((unsigned) (dsPtr->length + 1));
+	    dsPtr->string = (char *) ckalloc((unsigned) (dsPtr->length + 1));
 	    dsPtr->spaceAvl = dsPtr->length + 1;
 	}
 	strcpy(dsPtr->string, iPtr->result);
